@@ -64,7 +64,7 @@ struct _EphyDownload {
   gboolean was_moved;
 };
 
-G_DEFINE_TYPE (EphyDownload, ephy_download, G_TYPE_OBJECT)
+G_DEFINE_FINAL_TYPE (EphyDownload, ephy_download, G_TYPE_OBJECT)
 
 enum {
   PROP_0,
@@ -413,7 +413,7 @@ ephy_download_failed (EphyDownload  *download,
  * Executes the given @action for @download, this can be any of
  * #EphyDownloadActionType.
  *
- * Returns: %TRUE if the action was executed succesfully.
+ * Returns: %TRUE if the action was executed successfully.
  *
  **/
 gboolean
