@@ -35,10 +35,10 @@
 #include "ephy-settings.h"
 #include "ephy-string.h"
 #include "ephy-web-view.h"
-#include "nautilus-floating-bar.h"
+#include "ephy-floating-bar.h"
 
 #include <glib/gi18n.h>
-#include <webkit2/webkit2.h>
+#include <webkit/webkit.h>
 
 static void     ephy_embed_constructed (GObject *object);
 static void     ephy_embed_restored_window_cb (EphyEmbedShell *shell,
@@ -59,7 +59,7 @@ struct _EphyEmbed {
   EphyFindToolbar *find_toolbar;
   GtkBox *top_widgets_vbox;
   WebKitWebView *web_view;
-  GSList *destroy_on_transition_list;
+  GSList *destroy_on_transition_list; /* unowned GtkWidgets */
   GtkWidget *overlay;
   GtkWidget *floating_bar;
   GtkWidget *progress;
@@ -70,8 +70,8 @@ struct _EphyEmbed {
   WebKitWebViewSessionState *delayed_state;
   guint delayed_request_source_id;
 
-  GSList *messages;
-  GSList *keys;
+  GSList *messages; /* owned EphyEmbedStatusbarMsgs */
+  GSList *keys;     /* owned strings */
 
   guint seq_context_id;
   guint seq_message_id;
@@ -89,7 +89,7 @@ struct _EphyEmbed {
   gboolean progress_bar_enabled;
 };
 
-G_DEFINE_TYPE (EphyEmbed, ephy_embed, GTK_TYPE_BOX)
+G_DEFINE_FINAL_TYPE (EphyEmbed, ephy_embed, GTK_TYPE_BOX)
 
 enum {
   PROP_0,
@@ -134,7 +134,7 @@ static guint
 ephy_embed_statusbar_get_context_id (EphyEmbed  *embed,
                                      const char *context_description)
 {
-  char *string;
+  g_autofree char *string = NULL;
   guint id;
 
   g_assert (EPHY_IS_EMBED (embed));
@@ -147,9 +147,8 @@ ephy_embed_statusbar_get_context_id (EphyEmbed  *embed,
   if (id == 0) {
     id = embed->seq_context_id++;
     g_object_set_data_full (G_OBJECT (embed), string, GUINT_TO_POINTER (id), NULL);
-    embed->keys = g_slist_prepend (embed->keys, string);
-  } else
-    g_free (string);
+    embed->keys = g_slist_prepend (embed->keys, g_steal_pointer (&string));
+  }
 
   return id;
 }
@@ -158,13 +157,14 @@ static void
 ephy_embed_set_statusbar_label (EphyEmbed  *embed,
                                 const char *label)
 {
-  nautilus_floating_bar_set_primary_label (NAUTILUS_FLOATING_BAR (embed->floating_bar), label);
+  ephy_floating_bar_set_primary_label (EPHY_FLOATING_BAR (embed->floating_bar), label);
 
   if (label == NULL || label[0] == '\0') {
-    gtk_widget_hide (embed->floating_bar);
+    gtk_widget_set_visible (embed->floating_bar, FALSE);
     gtk_widget_set_halign (embed->floating_bar, GTK_ALIGN_START);
+    gtk_widget_remove_css_class (embed->floating_bar, "end");
   } else
-    gtk_widget_show (embed->floating_bar);
+    gtk_widget_set_visible (embed->floating_bar, TRUE);
 }
 
 static void
@@ -245,7 +245,7 @@ ephy_embed_destroy_top_widgets (EphyEmbed *embed)
 
   for (iter = embed->destroy_on_transition_list; iter; iter = iter->next) {
     g_signal_handlers_disconnect_by_func (iter->data, remove_from_destroy_list_cb, embed);
-    gtk_widget_destroy (GTK_WIDGET (iter->data));
+    gtk_box_remove (embed->top_widgets_vbox, GTK_WIDGET (iter->data));
   }
 
   embed->destroy_on_transition_list = NULL;
@@ -310,7 +310,7 @@ load_changed_cb (WebKitWebView   *web_view,
   }
 }
 
-static void
+static gboolean
 ephy_embed_grab_focus (GtkWidget *widget)
 {
   GtkWidget *child;
@@ -318,7 +318,9 @@ ephy_embed_grab_focus (GtkWidget *widget)
   child = GTK_WIDGET (ephy_embed_get_web_view (EPHY_EMBED (widget)));
 
   if (child)
-    gtk_widget_grab_focus (child);
+    return gtk_widget_grab_focus (child);
+
+  return FALSE;
 }
 
 
@@ -326,7 +328,7 @@ static gboolean
 fullscreen_message_label_hide (EphyEmbed *embed)
 {
   if (embed->fullscreen_message_id) {
-    gtk_widget_hide (embed->fullscreen_message_label);
+    gtk_widget_set_visible (embed->fullscreen_message_label, FALSE);
     g_source_remove (embed->fullscreen_message_id);
     embed->fullscreen_message_id = 0;
   }
@@ -338,7 +340,7 @@ void
 ephy_embed_entering_fullscreen (EphyEmbed *embed)
 {
   if (!g_settings_get_boolean (EPHY_SETTINGS_LOCKDOWN, EPHY_PREFS_LOCKDOWN_FULLSCREEN)) {
-    gtk_widget_show (embed->fullscreen_message_label);
+    gtk_widget_set_visible (embed->fullscreen_message_label, TRUE);
 
     g_clear_handle_id (&embed->fullscreen_message_id, g_source_remove);
     embed->fullscreen_message_id = g_timeout_add_seconds (5,
@@ -373,39 +375,21 @@ ephy_embed_dispose (GObject *object)
   G_OBJECT_CLASS (ephy_embed_parent_class)->dispose (object);
 }
 
+static void
+statusbar_message_free (EphyEmbedStatusbarMsg *message)
+{
+  g_free (message->text);
+  g_free (message);
+}
+
 static void
 ephy_embed_finalize (GObject *object)
 {
   EphyEmbed *embed = EPHY_EMBED (object);
-  EphyEmbedShell *shell = ephy_embed_shell_get_default ();
-  GSList *list;
 
-  g_signal_handlers_disconnect_by_func (shell, ephy_embed_restored_window_cb, embed);
-
-  list = embed->destroy_on_transition_list;
-  for (; list; list = list->next) {
-    GtkWidget *widget = GTK_WIDGET (list->data);
-    g_signal_handlers_disconnect_by_func (widget, remove_from_destroy_list_cb, embed);
-  }
   g_slist_free (embed->destroy_on_transition_list);
-
-  for (list = embed->messages; list; list = list->next) {
-    EphyEmbedStatusbarMsg *msg;
-
-    msg = list->data;
-    g_free (msg->text);
-    g_free (msg);
-  }
-
-  g_slist_free (embed->messages);
-  embed->messages = NULL;
-
-  for (list = embed->keys; list; list = list->next)
-    g_free (list->data);
-
-  g_slist_free (embed->keys);
-  embed->keys = NULL;
-
+  g_slist_free_full (embed->messages, (GDestroyNotify)statusbar_message_free);
+  g_slist_free_full (embed->keys, g_free);
   g_free (embed->title);
 
   G_OBJECT_CLASS (ephy_embed_parent_class)->finalize (object);
@@ -483,22 +467,19 @@ ephy_embed_class_init (EphyEmbedClass *klass)
 
   obj_properties[PROP_WEB_VIEW] =
     g_param_spec_object ("web-view",
-                         "Web View",
-                         "The WebView contained in the embed",
+                         NULL, NULL,
                          EPHY_TYPE_WEB_VIEW,
                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
 
   obj_properties[PROP_TITLE] =
     g_param_spec_string ("title",
-                         "Title",
-                         "The embed's title",
+                         NULL, NULL,
                          NULL,
                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
 
   obj_properties[PROP_PROGRESS_BAR_ENABLED] =
     g_param_spec_boolean ("progress-bar-enabled",
-                          "Progress bar",
-                          "Whether to show progress bar within embed",
+                          NULL, NULL,
                           TRUE,
                           G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
 
@@ -588,7 +569,7 @@ status_message_notify_cb (EphyWebView *view,
 static gboolean
 clear_progress_cb (EphyEmbed *embed)
 {
-  gtk_widget_hide (embed->progress);
+  gtk_widget_set_visible (embed->progress, FALSE);
   embed->clear_progress_source_id = 0;
 
   return FALSE;
@@ -608,7 +589,7 @@ progress_update (EphyWebView *view,
   uri = webkit_web_view_get_uri (embed->web_view);
   if (!uri || g_str_has_prefix (uri, "ephy-about:") ||
       g_str_has_prefix (uri, "about:")) {
-    gtk_widget_hide (embed->progress);
+    gtk_widget_set_visible (embed->progress, FALSE);
     return;
   }
 
@@ -621,13 +602,13 @@ progress_update (EphyWebView *view,
                                                      embed);
     g_source_set_name_by_id (embed->clear_progress_source_id, "[epiphany] clear_progress_cb");
   } else
-    gtk_widget_show (embed->progress);
+    gtk_widget_set_visible (embed->progress, TRUE);
 
   gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (embed->progress),
                                  (loading || progress == 1.0) ? progress : 0.0);
 }
 
-static gboolean
+static void
 load_delayed_request_if_mapped (gpointer user_data)
 {
   EphyEmbed *embed = EPHY_EMBED (user_data);
@@ -637,7 +618,7 @@ load_delayed_request_if_mapped (gpointer user_data)
   embed->delayed_request_source_id = 0;
 
   if (!gtk_widget_get_mapped (GTK_WIDGET (embed)))
-    return G_SOURCE_REMOVE;
+    return;
 
   web_view = ephy_embed_get_web_view (embed);
   if (embed->delayed_state)
@@ -652,7 +633,12 @@ load_delayed_request_if_mapped (gpointer user_data)
   g_clear_object (&embed->delayed_request);
   g_clear_pointer (&embed->delayed_state, webkit_web_view_session_state_unref);
 
-  return G_SOURCE_REMOVE;
+  /* We have a binding to `is-loading` in `ephy_tab_view_add_tab ()` that depends on
+   * whether the page is a placeholder (to avoid showing the spinner on tabs while restoring the session),
+   * so after removing the placeholder we need to notify it again. */
+  g_object_notify (G_OBJECT (web_view), "is-loading");
+
+  return;
 }
 
 static void
@@ -665,7 +651,7 @@ ephy_embed_maybe_load_delayed_request (EphyEmbed *embed)
    * is scrolling rapidly through a bunch of delayed tabs, we don't start
    * loading them all.
    */
-  embed->delayed_request_source_id = g_timeout_add (300, load_delayed_request_if_mapped, embed);
+  embed->delayed_request_source_id = g_timeout_add_once (300, load_delayed_request_if_mapped, embed);
   g_source_set_name_by_id (embed->delayed_request_source_id, "[epiphany] load_delayed_request_if_mapped");
 }
 
@@ -686,15 +672,47 @@ ephy_embed_mapped_cb (GtkWidget *widget,
   ephy_embed_maybe_load_delayed_request ((EphyEmbed *)widget);
 }
 
+static void
+floating_bar_motion_cb (GtkEventControllerMotion *self,
+                        double                    x,
+                        double                    y,
+                        EphyEmbed                *embed)
+{
+  graphene_rect_t bounds;
+
+  if (!gtk_widget_get_visible (embed->floating_bar))
+    return;
+
+  g_assert (gtk_widget_compute_bounds (embed->floating_bar,
+                                       GTK_WIDGET (embed),
+                                       &bounds));
+
+  if (!gtk_widget_contains (embed->floating_bar,
+                            x - bounds.origin.x,
+                            y - bounds.origin.y)) {
+    return;
+  }
+
+  if (gtk_widget_get_halign (embed->floating_bar) == GTK_ALIGN_START) {
+    gtk_widget_set_halign (embed->floating_bar, GTK_ALIGN_END);
+    gtk_widget_add_css_class (embed->floating_bar, "end");
+    gtk_widget_queue_allocate (GTK_WIDGET (embed->overlay));
+  } else {
+    gtk_widget_set_halign (embed->floating_bar, GTK_ALIGN_START);
+    gtk_widget_remove_css_class (embed->floating_bar, "end");
+  }
+}
+
 static void
 ephy_embed_constructed (GObject *object)
 {
   EphyEmbed *embed = (EphyEmbed *)object;
   EphyEmbedShell *shell = ephy_embed_shell_get_default ();
   WebKitWebInspector *inspector;
+  GtkEventController *controller;
 
-  g_signal_connect (shell, "window-restored",
-                    G_CALLBACK (ephy_embed_restored_window_cb), embed);
+  g_signal_connect_object (shell, "window-restored",
+                           G_CALLBACK (ephy_embed_restored_window_cb), embed, 0);
 
   g_signal_connect (embed, "map",
                     G_CALLBACK (ephy_embed_mapped_cb), NULL);
@@ -702,95 +720,85 @@ ephy_embed_constructed (GObject *object)
   /* Skeleton */
   embed->overlay = gtk_overlay_new ();
 
-  gtk_widget_add_events (embed->overlay,
-                         GDK_ENTER_NOTIFY_MASK |
-                         GDK_LEAVE_NOTIFY_MASK);
-  gtk_container_add (GTK_CONTAINER (embed->overlay), GTK_WIDGET (embed->web_view));
+  gtk_widget_set_vexpand (embed->overlay, TRUE);
+  gtk_overlay_set_child (GTK_OVERLAY (embed->overlay), GTK_WIDGET (embed->web_view));
 
   /* Floating message popup for fullscreen mode. */
   embed->fullscreen_message_label = gtk_label_new (NULL);
   gtk_widget_set_name (embed->fullscreen_message_label, "fullscreen-popup");
   gtk_widget_set_halign (embed->fullscreen_message_label, GTK_ALIGN_CENTER);
   gtk_widget_set_valign (embed->fullscreen_message_label, GTK_ALIGN_CENTER);
-  gtk_widget_set_no_show_all (embed->fullscreen_message_label, TRUE);
+  gtk_widget_set_can_target (embed->fullscreen_message_label, FALSE);
+  gtk_widget_set_visible (embed->fullscreen_message_label, FALSE);
   gtk_overlay_add_overlay (GTK_OVERLAY (embed->overlay), embed->fullscreen_message_label);
   ephy_embed_set_fullscreen_message (embed, FALSE);
 
   /* statusbar is hidden by default */
-  embed->floating_bar = nautilus_floating_bar_new (NULL, NULL, FALSE);
+  embed->floating_bar = ephy_floating_bar_new ();
   gtk_widget_set_halign (embed->floating_bar, GTK_ALIGN_START);
   gtk_widget_set_valign (embed->floating_bar, GTK_ALIGN_END);
-  gtk_widget_set_no_show_all (embed->floating_bar, TRUE);
+  gtk_widget_set_visible (embed->floating_bar, FALSE);
 
   gtk_overlay_add_overlay (GTK_OVERLAY (embed->overlay), embed->floating_bar);
-  gtk_overlay_set_overlay_pass_through (GTK_OVERLAY (embed->overlay), embed->floating_bar, TRUE);
 
   if (embed->progress_bar_enabled) {
     embed->progress = gtk_progress_bar_new ();
-    gtk_style_context_add_class (gtk_widget_get_style_context (embed->progress),
-                                 GTK_STYLE_CLASS_OSD);
+    gtk_widget_add_css_class (embed->progress, "osd");
     gtk_widget_set_halign (embed->progress, GTK_ALIGN_FILL);
     gtk_widget_set_valign (embed->progress, GTK_ALIGN_START);
     gtk_overlay_add_overlay (GTK_OVERLAY (embed->overlay), embed->progress);
   }
 
   embed->find_toolbar = ephy_find_toolbar_new (embed->web_view);
-  g_signal_connect (embed->find_toolbar, "close",
-                    G_CALLBACK (ephy_embed_find_toolbar_close_cb),
-                    embed);
+  g_signal_connect_object (embed->find_toolbar, "close",
+                           G_CALLBACK (ephy_embed_find_toolbar_close_cb),
+                           embed, 0);
 
-  gtk_box_pack_start (GTK_BOX (embed),
-                      GTK_WIDGET (embed->find_toolbar),
-                      FALSE, FALSE, 0);
+  gtk_box_append (GTK_BOX (embed), GTK_WIDGET (embed->find_toolbar));
 
   if (embed->progress_bar_enabled)
-    embed->progress_update_handler_id = g_signal_connect (embed->web_view, "notify::estimated-load-progress",
-                                                          G_CALLBACK (progress_update), object);
+    embed->progress_update_handler_id = g_signal_connect_object (embed->web_view, "notify::estimated-load-progress",
+                                                                 G_CALLBACK (progress_update), embed, 0);
 
-  gtk_box_pack_start (GTK_BOX (embed),
-                      GTK_WIDGET (embed->top_widgets_vbox),
-                      FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (embed), embed->overlay, TRUE, TRUE, 0);
+  gtk_box_append (GTK_BOX (embed), GTK_WIDGET (embed->top_widgets_vbox));
+  gtk_box_append (GTK_BOX (embed), GTK_WIDGET (embed->overlay));
 
-  gtk_widget_show (GTK_WIDGET (embed->top_widgets_vbox));
-  gtk_widget_show (GTK_WIDGET (embed->web_view));
-  gtk_widget_show_all (embed->overlay);
+  g_signal_connect_object (embed->web_view, "notify::title",
+                           G_CALLBACK (web_view_title_changed_cb), embed, 0);
+  g_signal_connect_object (embed->web_view, "load-changed",
+                           G_CALLBACK (load_changed_cb), embed, 0);
+  g_signal_connect_object (embed->web_view, "enter-fullscreen",
+                           G_CALLBACK (entering_fullscreen_cb), embed, 0);
+  g_signal_connect_object (embed->web_view, "leave-fullscreen",
+                           G_CALLBACK (leaving_fullscreen_cb), embed, 0);
 
-  g_object_connect (embed->web_view,
-                    "signal::notify::title", G_CALLBACK (web_view_title_changed_cb), embed,
-                    "signal::load-changed", G_CALLBACK (load_changed_cb), embed,
-                    "signal::enter-fullscreen", G_CALLBACK (entering_fullscreen_cb), embed,
-                    "signal::leave-fullscreen", G_CALLBACK (leaving_fullscreen_cb), embed,
-                    NULL);
-
-  embed->status_handler_id = g_signal_connect (embed->web_view, "notify::status-message",
-                                               G_CALLBACK (status_message_notify_cb),
-                                               embed);
+  embed->status_handler_id = g_signal_connect_object (embed->web_view, "notify::status-message",
+                                                      G_CALLBACK (status_message_notify_cb),
+                                                      embed, 0);
 
   /* The inspector */
   inspector = webkit_web_view_get_inspector (embed->web_view);
 
-  g_signal_connect (inspector, "attach",
-                    G_CALLBACK (ephy_embed_attach_inspector_cb),
-                    embed);
-  g_signal_connect (inspector, "closed",
-                    G_CALLBACK (ephy_embed_close_inspector_cb),
-                    embed);
+  g_signal_connect_object (inspector, "attach",
+                           G_CALLBACK (ephy_embed_attach_inspector_cb),
+                           embed, 0);
+  g_signal_connect_object (inspector, "closed",
+                           G_CALLBACK (ephy_embed_close_inspector_cb),
+                           embed, 0);
 
   if (webkit_web_view_is_controlled_by_automation (embed->web_view)) {
-    GtkWidget *info_bar;
-    GtkWidget *label;
+    GtkWidget *banner;
 
-    info_bar = gtk_info_bar_new ();
-    gtk_info_bar_set_message_type (GTK_INFO_BAR (info_bar), GTK_MESSAGE_INFO);
     /* Translators: this means WebDriver control. */
-    label = gtk_label_new (_("Web is being controlled by automation."));
-    gtk_box_pack_start (GTK_BOX (gtk_info_bar_get_content_area (GTK_INFO_BAR (info_bar))), label, FALSE, FALSE, 0);
-    gtk_widget_show (label);
+    banner = adw_banner_new (_("Web is being controlled by automation"));
+    adw_banner_set_revealed (ADW_BANNER (banner), TRUE);
 
-    ephy_embed_add_top_widget (embed, info_bar, EPHY_EMBED_TOP_WIDGET_POLICY_RETAIN_ON_TRANSITION);
-    gtk_widget_show (info_bar);
+    ephy_embed_add_top_widget (embed, banner, EPHY_EMBED_TOP_WIDGET_POLICY_RETAIN_ON_TRANSITION);
   }
+
+  controller = gtk_event_controller_motion_new ();
+  g_signal_connect (controller, "motion", G_CALLBACK (floating_bar_motion_cb), embed);
+  gtk_widget_add_controller (GTK_WIDGET (embed), controller);
 }
 
 static void
@@ -860,11 +868,10 @@ ephy_embed_add_top_widget (EphyEmbed                *embed,
     list = g_slist_prepend (list, widget);
     embed->destroy_on_transition_list = list;
 
-    g_signal_connect (widget, "destroy", G_CALLBACK (remove_from_destroy_list_cb), embed);
+    g_signal_connect_object (widget, "destroy", G_CALLBACK (remove_from_destroy_list_cb), embed, 0);
   }
 
-  gtk_box_pack_end (embed->top_widgets_vbox,
-                    GTK_WIDGET (widget), FALSE, FALSE, 0);
+  gtk_box_prepend (embed->top_widgets_vbox, widget);
 }
 
 /**
@@ -890,8 +897,7 @@ ephy_embed_remove_top_widget (EphyEmbed *embed,
     embed->destroy_on_transition_list = list;
   }
 
-  gtk_container_remove (GTK_CONTAINER (embed->top_widgets_vbox),
-                        GTK_WIDGET (widget));
+  gtk_box_remove (embed->top_widgets_vbox, widget);
 }
 
 /**
@@ -985,6 +991,14 @@ ephy_embed_detach_notification_container (EphyEmbed *embed)
      * notification widget, removing it from the container will destroy the
      * singleton. To prevent this, add a reference to it before removing it
      * from the container. */
-    gtk_container_remove (GTK_CONTAINER (embed->overlay), g_object_ref (GTK_WIDGET (container)));
+    gtk_overlay_remove_overlay (GTK_OVERLAY (embed->overlay), g_object_ref (GTK_WIDGET (container)));
   }
 }
+
+WebKitWebViewSessionState *
+ephy_embed_get_session_state (EphyEmbed *embed)
+{
+  if (embed->delayed_state)
+    return webkit_web_view_session_state_ref (embed->delayed_state);
+  return webkit_web_view_get_session_state (embed->web_view);
+}
