@@ -20,6 +20,12 @@ Ephy.getAppleMobileWebAppCapable = function()
     return false;
 };
 
+Ephy.getWebAppManifestURL = function()
+{
+    const manifest = document.head.querySelector('link[rel=manifest]');
+    return manifest ? manifest.href : null;
+};
+
 Ephy.getWebAppTitle = function()
 {
     const metas = document.getElementsByTagName('meta');
@@ -422,8 +428,7 @@ Ephy.PasswordManager = class PasswordManager
         });
     }
 
-    // FIXME: Why is pageID a parameter here?
-    requestSave(origin, targetOrigin, username, password, usernameField, passwordField, isNew, pageID)
+    requestSave(origin, targetOrigin, username, password, usernameField, passwordField, isNew)
     {
         if (Ephy.isSandboxedWebContent()) {
             Ephy.log(`Not requesting to save password for origin=${origin} because web content is sandboxed`);
@@ -434,7 +439,7 @@ Ephy.PasswordManager = class PasswordManager
 
         window.webkit.messageHandlers.passwordManagerRequestSave.postMessage({
             origin, targetOrigin, username, password, usernameField, passwordField, isNew,
-            pageID
+            pageID: this._pageID
         });
     }
 
@@ -474,6 +479,7 @@ Ephy.FormManager = class FormManager
         this._passwordFormMessageSerializer = null;
         this._preFillUserMenu = null;
         this._elementBeingAutoFilled = null;
+        this._submissionHandled = false;
     }
 
     // Public
@@ -569,6 +575,11 @@ Ephy.FormManager = class FormManager
         if (!Ephy.shouldRememberPasswords())
             return;
 
+        if (this._submissionHandled)
+            return;
+
+        this._submissionHandled = true;
+
         const formAuth = this._generateFormAuth(false);
         if (!formAuth || !formAuth.password)
             return;
@@ -617,8 +628,7 @@ Ephy.FormManager = class FormManager
                                                  formAuth.password,
                                                  formAuth.usernameField,
                                                  formAuth.passwordField,
-                                                 authInfo === null,
-                                                 this._pageID);
+                                                 authInfo === null);
             }
         );
     }
@@ -637,14 +647,37 @@ Ephy.FormManager = class FormManager
         return false;
     }
 
+    _getFormAction()
+    {
+        // We used to naively access this._form.action to get the action
+        // attribute of the form, which works 99% of the time, but fails if any
+        // input element of the form is named "action". We don't want to get
+        // that by mistake, so we can't rely on the JavaScript property. See:
+        // https://gitlab.gnome.org/GNOME/epiphany/-/issues/2114
+        // https://stackoverflow.com/questions/18100630/
+        //
+        // One disadvantage of manually querying the attribute value rather
+        // than directly accessing the property is the value may be a relative
+        // path rather than a URL, so let's always convert to URL here.
+        //
+        // This may not actually be the real action of the form submission,
+        // because the submitter element gets to override the action. However,
+        // we don't know which element will be submitted yet, so we cannot
+        // consider that for autofill purposes. It's OK because the target URL
+        // we save into the password manager is only a heuristic.
+
+        const action = this._form.getAttribute('action');
+        return action ? new URL(action, window.location) : null;
+    }
+
     _passwordFormFocused(event)
     {
         let isFormActionInsecure = false;
-        if (this._form.action) {
-            const url = new URL(this._form.action);
-            if (url.protocol === 'http:') {
+        const actionURL = this._getFormAction();
+        if (actionURL) {
+            if (actionURL.protocol === 'http:') {
                 // We trust localhost to be local since glib!616.
-                const parts = url.hostname.split('.');
+                const parts = actionURL.hostname.split('.');
                 if (parts.length > 0) {
                     const tld = parts[parts.length - 1];
                     isFormActionInsecure = tld !== '127.0.0.1' && tld !== '::1' && tld !== 'localhost';
@@ -761,7 +794,7 @@ Ephy.FormManager = class FormManager
 
         formAuth.origin = new URL(String(window.location)).origin;
         try {
-            formAuth.targetOrigin = new URL(this._form.action).origin;
+            formAuth.targetOrigin = this._getFormAction().origin;
         } catch {
             formAuth.targetOrigin = formAuth.origin;
         }
@@ -802,4 +835,4 @@ window.document.addEventListener('contextmenu', (event) => {
     // isContentEditable is always false, in practice this seems functional enough.
     contextMenuElementIsEditable = event.target.tagName.toLowerCase() === 'input';
     contextMenuElementIsPassword = event.target.type === 'password';
-});
\ No newline at end of file
+});
