@@ -30,6 +30,21 @@
 #include <inttypes.h>
 #include <stdio.h>
 
+void
+ephy_password_request_data_free (EphyPasswordRequestData *request_data)
+{
+  g_assert (request_data);
+
+  g_free (request_data->origin);
+  g_free (request_data->target_origin);
+  g_free (request_data->username);
+  g_free (request_data->password);
+  g_free (request_data->usernameField);
+  g_free (request_data->passwordField);
+
+  g_free (request_data);
+}
+
 const SecretSchema *
 ephy_password_manager_get_password_schema (void)
 {
@@ -62,9 +77,9 @@ static void ephy_password_manager_forget_record (EphyPasswordManager *self,
 
 static void ephy_synchronizable_manager_iface_init (EphySynchronizableManagerInterface *iface);
 
-G_DEFINE_TYPE_WITH_CODE (EphyPasswordManager, ephy_password_manager, G_TYPE_OBJECT,
-                         G_IMPLEMENT_INTERFACE (EPHY_TYPE_SYNCHRONIZABLE_MANAGER,
-                                                ephy_synchronizable_manager_iface_init))
+G_DEFINE_FINAL_TYPE_WITH_CODE (EphyPasswordManager, ephy_password_manager, G_TYPE_OBJECT,
+                               G_IMPLEMENT_INTERFACE (EPHY_TYPE_SYNCHRONIZABLE_MANAGER,
+                                                      ephy_synchronizable_manager_iface_init))
 
 typedef struct {
   EphyPasswordManagerQueryCallback callback;
@@ -384,7 +399,7 @@ secret_password_store_cb (GObject               *source_object,
   secret_password_store_finish (result, &error);
   if (error) {
     if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
-      g_warning ("Failed to store password record for (%s, %s, %s, %s, %s): %s",
+      g_warning ("Failed to store password record for (%s, %s, %s, %s, %s) (is the secret service or secrets portal broken?): %s",
                  origin,
                  ephy_password_record_get_target_origin (data->record),
                  username,
@@ -581,7 +596,7 @@ retrieve_secret_cb (GObject        *source_object,
   value = secret_retrievable_retrieve_secret_finish (retrievable, result, &error);
   if (!value) {
     if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
-      g_warning ("Failed to retrieve password: %s", error->message);
+      g_warning ("Failed to retrieve password (is the secret service or secrets portal broken?): %s", error->message);
     g_error_free (error);
     goto out;
   }
@@ -643,7 +658,7 @@ secret_password_search_cb (GObject        *source_object,
   if (!matches) {
     if (error) {
       if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
-        g_warning ("Failed to search secrets in password schema: %s", error->message);
+        g_warning ("Failed to search secret storage (is the secret service or secrets portal broken?): %s", error->message);
       g_error_free (error);
     }
     if (data->callback)
@@ -742,7 +757,7 @@ secret_password_clear_cb (GObject      *source_object,
     if (data && data->task)
       g_task_return_error (data->task, error);
     else if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
-      g_warning ("Failed to clear secrets from password schema: %s", error->message);
+      g_warning ("Failed to clear secrets (is the secret service or secrets portal broken?): %s", error->message);
     g_clear_pointer (&data, manage_record_async_data_free);
     return;
   }
