@@ -22,6 +22,7 @@
 
 #include "ephy-bookmark-properties.h"
 #include "ephy-bookmark-row.h"
+#include "ephy-bookmarks-popover.h"
 #include "ephy-embed-prefs.h"
 #include "ephy-embed-shell.h"
 #include "ephy-favicon-helpers.h"
@@ -31,13 +32,14 @@ struct _EphyBookmarkRow {
   GtkListBoxRow parent_instance;
 
   EphyBookmark *bookmark;
+  GCancellable *cancellable;
 
   GtkWidget *favicon_image;
   GtkWidget *title_label;
   GtkWidget *properties_button;
 };
 
-G_DEFINE_TYPE (EphyBookmarkRow, ephy_bookmark_row, GTK_TYPE_LIST_BOX_ROW)
+G_DEFINE_FINAL_TYPE (EphyBookmarkRow, ephy_bookmark_row, GTK_TYPE_LIST_BOX_ROW)
 
 enum {
   PROP_0,
@@ -51,31 +53,57 @@ static void
 ephy_bookmark_row_button_clicked_cb (EphyBookmarkRow *row,
                                      GtkButton       *button)
 {
+  GtkWidget *popover;
   GtkWidget *dialog;
-  GtkWidget *content_area;
   GtkWidget *grid;
+  GtkWidget *toolbar_view;
+  GtkEventController *controller;
+  GtkShortcut *shortcut;
 
   g_assert (EPHY_IS_BOOKMARK_ROW (row));
   g_assert (GTK_IS_BUTTON (button));
 
-  dialog = g_object_new (GTK_TYPE_DIALOG,
+  dialog = g_object_new (ADW_TYPE_WINDOW,
                          "title", _("Bookmark Properties"),
-                         "transient-for", GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (row))),
-                         "use-header-bar", TRUE,
+                         "transient-for", GTK_WINDOW (gtk_widget_get_root (GTK_WIDGET (row))),
                          "modal", TRUE,
                          NULL);
 
-  content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
+  shortcut = gtk_shortcut_new (gtk_keyval_trigger_new (GDK_KEY_Escape, 0),
+                               gtk_named_action_new ("window.close"));
+  controller = gtk_shortcut_controller_new ();
+
+  gtk_shortcut_controller_add_shortcut (GTK_SHORTCUT_CONTROLLER (controller), shortcut);
+
+  gtk_widget_add_controller (dialog, controller);
+
+  popover = gtk_widget_get_ancestor (GTK_WIDGET (row), GTK_TYPE_POPOVER);
+
+  if (popover)
+    gtk_popover_popdown (GTK_POPOVER (popover));
 
   grid = ephy_bookmark_properties_new (ephy_bookmark_row_get_bookmark (row),
-                                       EPHY_BOOKMARK_PROPERTIES_TYPE_DIALOG,
-                                       dialog);
-  gtk_window_set_default (GTK_WINDOW (dialog),
-                          ephy_bookmark_properties_get_add_tag_button (EPHY_BOOKMARK_PROPERTIES (grid)));
+                                       EPHY_BOOKMARK_PROPERTIES_TYPE_DIALOG);
+  gtk_window_set_default_widget (GTK_WINDOW (dialog),
+                                 ephy_bookmark_properties_get_add_tag_button (EPHY_BOOKMARK_PROPERTIES (grid)));
 
-  gtk_container_add (GTK_CONTAINER (content_area), grid);
+  toolbar_view = adw_toolbar_view_new ();
+  adw_toolbar_view_add_top_bar (ADW_TOOLBAR_VIEW (toolbar_view),
+                                adw_header_bar_new ());
+  adw_toolbar_view_set_content (ADW_TOOLBAR_VIEW (toolbar_view),
+                                grid);
+  adw_window_set_content (ADW_WINDOW (dialog), toolbar_view);
 
-  gtk_widget_show (dialog);
+  gtk_window_present (GTK_WINDOW (dialog));
+}
+
+void
+ephy_bookmark_row_open (EphyBookmarkRow *self,
+                        EphyLinkFlags    flags)
+{
+  GtkWidget *window = gtk_widget_get_ancestor (GTK_WIDGET (self), EPHY_TYPE_WINDOW);
+  const char *url = ephy_bookmark_get_url (self->bookmark);
+  ephy_link_open (EPHY_LINK (window), url, NULL, flags | EPHY_LINK_BOOKMARK);
 }
 
 static void
@@ -83,25 +111,22 @@ ephy_bookmark_row_favicon_loaded_cb (GObject      *source,
                                      GAsyncResult *result,
                                      gpointer      user_data)
 {
-  g_autoptr (EphyBookmarkRow) self = user_data;
+  EphyBookmarkRow *self = user_data;
   WebKitFaviconDatabase *database = WEBKIT_FAVICON_DATABASE (source);
-  cairo_surface_t *icon_surface;
-  g_autoptr (GdkPixbuf) favicon = NULL;
+  g_autoptr (GdkTexture) icon_texture = NULL;
+  g_autoptr (GIcon) favicon = NULL;
+  int scale;
+
+  icon_texture = webkit_favicon_database_get_favicon_finish (database, result, NULL);
+  if (!icon_texture)
+    return;
 
   g_assert (EPHY_IS_BOOKMARK_ROW (self));
 
-  icon_surface = webkit_favicon_database_get_favicon_finish (database, result, NULL);
-  if (icon_surface) {
-    int scale = gtk_widget_get_scale_factor (self->favicon_image);
-
-    favicon = ephy_pixbuf_get_from_surface_scaled (icon_surface, FAVICON_SIZE * scale, FAVICON_SIZE * scale);
-    cairo_surface_destroy (icon_surface);
-  }
-
-  if (favicon) {
-    if (self->favicon_image != NULL)
-      gtk_image_set_from_gicon (GTK_IMAGE (self->favicon_image), G_ICON (favicon), GTK_ICON_SIZE_BUTTON);
-  }
+  scale = gtk_widget_get_scale_factor (self->favicon_image);
+  favicon = ephy_favicon_get_from_texture_scaled (icon_texture, FAVICON_SIZE * scale, FAVICON_SIZE * scale);
+  if (favicon && self->favicon_image)
+    gtk_image_set_from_gicon (GTK_IMAGE (self->favicon_image), favicon);
 }
 
 static void
@@ -145,14 +170,10 @@ ephy_bookmark_row_dispose (GObject *object)
 
   g_clear_object (&self->bookmark);
 
-  G_OBJECT_CLASS (ephy_bookmark_row_parent_class)->dispose (object);
-}
+  g_cancellable_cancel (self->cancellable);
+  g_clear_object (&self->cancellable);
 
-static void
-favicon_image_destroyed (EphyBookmarkRow *self,
-                         GtkWidget       *favicon_image)
-{
-  self->favicon_image = NULL;
+  G_OBJECT_CLASS (ephy_bookmark_row_parent_class)->dispose (object);
 }
 
 static gboolean
@@ -183,12 +204,27 @@ transform_bookmark_title (GBinding     *binding,
   return TRUE;
 }
 
+static void
+ephy_bookmark_row_map (GtkWidget *widget)
+{
+  EphyBookmarkRow *self = EPHY_BOOKMARK_ROW (widget);
+  EphyEmbedShell *shell = ephy_embed_shell_get_default ();
+  WebKitFaviconDatabase *database;
+
+  GTK_WIDGET_CLASS (ephy_bookmark_row_parent_class)->map (widget);
+
+  database = ephy_embed_shell_get_favicon_database (shell);
+  webkit_favicon_database_get_favicon (database,
+                                       ephy_bookmark_get_url (self->bookmark),
+                                       self->cancellable,
+                                       (GAsyncReadyCallback)ephy_bookmark_row_favicon_loaded_cb,
+                                       self);
+}
+
 static void
 ephy_bookmark_row_constructed (GObject *object)
 {
   EphyBookmarkRow *self = EPHY_BOOKMARK_ROW (object);
-  EphyEmbedShell *shell = ephy_embed_shell_get_default ();
-  WebKitFaviconDatabase *database;
 
   G_OBJECT_CLASS (ephy_bookmark_row_parent_class)->constructed (object);
 
@@ -204,20 +240,16 @@ ephy_bookmark_row_constructed (GObject *object)
                    self->properties_button,
                    "visible",
                    G_SETTINGS_BIND_INVERT_BOOLEAN);
+}
 
-  database = webkit_web_context_get_favicon_database (ephy_embed_shell_get_web_context (shell));
-  webkit_favicon_database_get_favicon (database,
-                                       ephy_bookmark_get_url (self->bookmark),
-                                       NULL,
-                                       (GAsyncReadyCallback)ephy_bookmark_row_favicon_loaded_cb,
-                                       g_object_ref (self));
-
-  /* Although we keep a ref to ourself during the favicon load, so we are
-   * guaranteed to remain a valid GObject, the widget hierarchy could still
-   * be destroyed before ephy_bookmark_favicon_loaded_cb() is called. Hence we
-   * need to keep track of whether self->favicon_image is still valid. */
-  g_signal_connect_object (self->favicon_image, "destroy",
-                           G_CALLBACK (favicon_image_destroyed), self, G_CONNECT_SWAPPED);
+static void
+ephy_bookmark_row_activate (GtkListBoxRow *row)
+{
+  GtkWidget *popover = gtk_widget_get_ancestor (GTK_WIDGET (row), EPHY_TYPE_BOOKMARKS_POPOVER);
+  if (popover) {
+    gtk_popover_popdown (GTK_POPOVER (popover));
+  }
+  ephy_bookmark_row_open (EPHY_BOOKMARK_ROW (row), 0);
 }
 
 static void
@@ -231,10 +263,13 @@ ephy_bookmark_row_class_init (EphyBookmarkRowClass *klass)
   object_class->dispose = ephy_bookmark_row_dispose;
   object_class->constructed = ephy_bookmark_row_constructed;
 
+  widget_class->map = ephy_bookmark_row_map;
+
+  GTK_LIST_BOX_ROW_CLASS (klass)->activate = ephy_bookmark_row_activate;
+
   obj_properties[PROP_BOOKMARK] =
     g_param_spec_object ("bookmark",
-                         "An EphyBookmark object",
-                         "The EphyBookmark shown by this widget",
+                         NULL, NULL,
                          EPHY_TYPE_BOOKMARK,
                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
 
@@ -256,6 +291,8 @@ ephy_bookmark_row_init (EphyBookmarkRow *self)
                            G_CALLBACK (ephy_bookmark_row_button_clicked_cb),
                            self,
                            G_CONNECT_SWAPPED);
+
+  self->cancellable = g_cancellable_new ();
 }
 
 GtkWidget *
