@@ -47,11 +47,11 @@ struct _EphyBookmarksManager {
 static void list_model_iface_init (GListModelInterface *iface);
 static void ephy_synchronizable_manager_iface_init (EphySynchronizableManagerInterface *iface);
 
-G_DEFINE_TYPE_WITH_CODE (EphyBookmarksManager, ephy_bookmarks_manager, G_TYPE_OBJECT,
-                         G_IMPLEMENT_INTERFACE (G_TYPE_LIST_MODEL,
-                                                list_model_iface_init)
-                         G_IMPLEMENT_INTERFACE (EPHY_TYPE_SYNCHRONIZABLE_MANAGER,
-                                                ephy_synchronizable_manager_iface_init))
+G_DEFINE_FINAL_TYPE_WITH_CODE (EphyBookmarksManager, ephy_bookmarks_manager, G_TYPE_OBJECT,
+                               G_IMPLEMENT_INTERFACE (G_TYPE_LIST_MODEL,
+                                                      list_model_iface_init)
+                               G_IMPLEMENT_INTERFACE (EPHY_TYPE_SYNCHRONIZABLE_MANAGER,
+                                                      ephy_synchronizable_manager_iface_init))
 
 enum {
   BOOKMARK_ADDED,
@@ -202,9 +202,8 @@ ephy_bookmarks_manager_class_init (EphyBookmarksManagerClass *klass)
                   G_SIGNAL_RUN_LAST,
                   0,
                   NULL, NULL, NULL,
-                  G_TYPE_NONE, 2,
-                  G_TYPE_STRING,
-                  G_TYPE_INT);
+                  G_TYPE_NONE, 1,
+                  G_TYPE_STRING);
 }
 
 static void
@@ -228,8 +227,10 @@ ephy_bookmarks_manager_init (EphyBookmarksManager *self)
 
   /* Create DB file if it doesn't already exists */
   if (!g_file_test (self->gvdb_filename, G_FILE_TEST_EXISTS)) {
-    if (!ephy_bookmarks_manager_save_sync (self, &error))
+    if (!ephy_bookmarks_manager_save_sync (self, &error)) {
+      g_assert (error);
       g_warning ("Failed to save bookmarks: %s", error->message);
+    }
   }
 
   ephy_bookmarks_import (self, self->gvdb_filename, NULL);
@@ -415,8 +416,9 @@ ephy_bookmarks_manager_remove_bookmark_internal (EphyBookmarksManager *self,
   g_object_unref (bookmark);
 }
 
-void ephy_bookmarks_manager_remove_bookmark (EphyBookmarksManager *self,
-                                             EphyBookmark         *bookmark)
+void
+ephy_bookmarks_manager_remove_bookmark (EphyBookmarksManager *self,
+                                        EphyBookmark         *bookmark)
 {
   g_assert (EPHY_IS_BOOKMARKS_MANAGER (self));
   g_assert (EPHY_IS_BOOKMARK (bookmark));
@@ -495,7 +497,6 @@ ephy_bookmarks_manager_delete_tag (EphyBookmarksManager *self,
                                    const char           *tag)
 {
   GSequenceIter *iter = NULL;
-  int position;
 
   g_assert (EPHY_IS_BOOKMARKS_MANAGER (self));
   g_assert (tag != NULL);
@@ -508,15 +509,12 @@ ephy_bookmarks_manager_delete_tag (EphyBookmarksManager *self,
                             (GCompareDataFunc)ephy_bookmark_tags_compare,
                             NULL);
   g_assert (iter != NULL);
-
-  position = g_sequence_iter_get_position (iter);
-
   g_sequence_remove (iter);
 
   /* Also remove the tag from each bookmark if they have it */
   g_sequence_foreach (self->bookmarks, (GFunc)ephy_bookmark_remove_tag, (gpointer)tag);
 
-  g_signal_emit (self, signals[TAG_DELETED], 0, tag, position);
+  g_signal_emit (self, signals[TAG_DELETED], 0, tag);
 }
 
 gboolean
@@ -585,6 +583,15 @@ ephy_bookmarks_manager_get_bookmarks_with_tag (EphyBookmarksManager *self,
   return bookmarks;
 }
 
+gboolean
+ephy_bookmarks_manager_has_bookmarks_with_tag (EphyBookmarksManager *self,
+                                               const char           *tag)
+{
+  g_autoptr (GSequence) bookmarks = ephy_bookmarks_manager_get_bookmarks_with_tag (self, tag);
+
+  return g_sequence_is_empty (bookmarks);
+}
+
 GSequence *
 ephy_bookmarks_manager_get_tags (EphyBookmarksManager *self)
 {
@@ -622,7 +629,7 @@ bookmarks_export_cb (GObject      *source_object,
 {
   EphyBookmarksManager *self = EPHY_BOOKMARKS_MANAGER (source_object);
   g_autoptr (GTask) task = user_data;
-  GError *error;
+  GError *error = NULL;
 
   if (!ephy_bookmarks_export_finish (self, result, &error)) {
     g_task_return_error (task, error);
