@@ -35,8 +35,39 @@
 
 #define FXA_IFRAME_URL "https://accounts.firefox.com/signin?service=sync&context=fx_desktop_v3"
 
+#define EPHY_TYPE_SYNC_FREQUENCY (ephy_sync_frequency_get_type ())
+
+G_DECLARE_FINAL_TYPE (EphySyncFrequency, ephy_sync_frequency, EPHY, SYNC_FREQUENCY, GObject);
+
+struct _EphySyncFrequency {
+  GObject parent_instance;
+  guint frequency;
+};
+
+G_DEFINE_FINAL_TYPE (EphySyncFrequency, ephy_sync_frequency, G_TYPE_OBJECT)
+
+static void
+ephy_sync_frequency_class_init (EphySyncFrequencyClass *klass)
+{
+}
+
+static void
+ephy_sync_frequency_init (EphySyncFrequency *self)
+{
+}
+
+static EphySyncFrequency *
+ephy_sync_frequency_new (guint frequency)
+{
+  EphySyncFrequency *self = g_object_new (EPHY_TYPE_SYNC_FREQUENCY, NULL);
+
+  self->frequency = frequency;
+
+  return self;
+}
+
 struct _EphyFirefoxSyncDialog {
-  HdyWindow parent_instance;
+  AdwWindow parent_instance;
 
   GtkWidget *sync_page_group;
   GtkWidget *sync_firefox_iframe_box;
@@ -44,10 +75,10 @@ struct _EphyFirefoxSyncDialog {
   GtkWidget *sync_firefox_account_group;
   GtkWidget *sync_firefox_account_row;
   GtkWidget *sync_options_group;
-  GtkWidget *sync_bookmarks_switch;
-  GtkWidget *sync_passwords_switch;
-  GtkWidget *sync_history_switch;
-  GtkWidget *sync_open_tabs_switch;
+  GtkWidget *sync_bookmarks_row;
+  GtkWidget *sync_passwords_row;
+  GtkWidget *sync_history_row;
+  GtkWidget *sync_open_tabs_row;
   GtkWidget *sync_frequency_row;
   GtkWidget *sync_now_button;
   GtkWidget *synced_tabs_button;
@@ -61,7 +92,7 @@ struct _EphyFirefoxSyncDialog {
   WebKitUserScript *fxa_script;
 };
 
-G_DEFINE_TYPE (EphyFirefoxSyncDialog, ephy_firefox_sync_dialog, HDY_TYPE_WINDOW)
+G_DEFINE_FINAL_TYPE (EphyFirefoxSyncDialog, ephy_firefox_sync_dialog, ADW_TYPE_WINDOW)
 
 static const guint sync_frequency_minutes[] = { 5, 15, 30, 60 };
 
@@ -74,13 +105,14 @@ sync_collection_toggled_cb (GtkWidget             *sw,
   EphyShell *shell = ephy_shell_get_default ();
   EphySyncService *service = ephy_shell_get_sync_service (shell);
 
-  if (GTK_WIDGET (sw) == sync_dialog->sync_bookmarks_switch) {
-    manager = EPHY_SYNCHRONIZABLE_MANAGER (ephy_shell_get_bookmarks_manager (shell));
-  } else if (GTK_WIDGET (sw) == sync_dialog->sync_passwords_switch) {
+  if (GTK_WIDGET (sw) == sync_dialog->sync_bookmarks_row) {
+    return; /* https://gitlab.gnome.org/GNOME/epiphany/-/issues/1118#note_1731178
+    manager = EPHY_SYNCHRONIZABLE_MANAGER (ephy_shell_get_bookmarks_manager (shell)); */
+  } else if (GTK_WIDGET (sw) == sync_dialog->sync_passwords_row) {
     manager = EPHY_SYNCHRONIZABLE_MANAGER (ephy_embed_shell_get_password_manager (EPHY_EMBED_SHELL (shell)));
-  } else if (GTK_WIDGET (sw) == sync_dialog->sync_history_switch) {
+  } else if (GTK_WIDGET (sw) == sync_dialog->sync_history_row) {
     manager = EPHY_SYNCHRONIZABLE_MANAGER (ephy_shell_get_history_manager (shell));
-  } else if (GTK_WIDGET (sw) == sync_dialog->sync_open_tabs_switch) {
+  } else if (GTK_WIDGET (sw) == sync_dialog->sync_open_tabs_row) {
     manager = EPHY_SYNCHRONIZABLE_MANAGER (ephy_shell_get_open_tabs_manager (shell));
     ephy_open_tabs_manager_clear_cache (EPHY_OPEN_TABS_MANAGER (manager));
   } else {
@@ -107,7 +139,7 @@ sync_set_last_sync_time (EphyFirefoxSyncDialog *sync_dialog)
      */
     char *text = g_strdup_printf (_("Last synchronized: %s"), time);
 
-    hdy_action_row_set_subtitle (HDY_ACTION_ROW (sync_dialog->sync_firefox_account_row), text);
+    adw_action_row_set_subtitle (ADW_ACTION_ROW (sync_dialog->sync_firefox_account_row), text);
 
     g_free (text);
     g_free (time);
@@ -162,11 +194,12 @@ sync_secrets_store_finished_cb (EphySyncService       *service,
   g_assert (EPHY_IS_FIREFOX_SYNC_DIALOG (sync_dialog));
 
   if (!error) {
-    hdy_preferences_row_set_title (HDY_PREFERENCES_ROW (sync_dialog->sync_firefox_account_row),
+    adw_preferences_row_set_title (ADW_PREFERENCES_ROW (sync_dialog->sync_firefox_account_row),
                                    ephy_sync_utils_get_sync_user ());
-    gtk_widget_hide (sync_dialog->sync_page_group);
-    gtk_widget_show (sync_dialog->sync_firefox_account_group);
-    gtk_widget_show (sync_dialog->sync_options_group);
+    gtk_widget_set_visible (sync_dialog->sync_page_group, FALSE);
+    gtk_widget_set_visible (sync_dialog->sync_firefox_account_group, TRUE);
+    gtk_widget_set_visible (sync_dialog->sync_options_group, TRUE);
+    gtk_widget_set_visible (sync_dialog->sync_now_button, TRUE);
   } else {
     /* Display the error message and reload the iframe. */
     sync_sign_in_details_show (sync_dialog, error->message);
@@ -211,7 +244,7 @@ sync_message_to_fxa_content (EphyFirefoxSyncDialog *sync_dialog,
                             type, detail_str);
 
   /* We don't expect any response from the server. */
-  webkit_web_view_run_javascript (sync_dialog->fxa_web_view, script, NULL, NULL, NULL);
+  webkit_web_view_evaluate_javascript (sync_dialog->fxa_web_view, script, -1, NULL, NULL, NULL, NULL, NULL);
 
   g_free (script);
   g_free (detail_str);
@@ -325,7 +358,7 @@ out_no_error:
 
 static void
 sync_message_from_fxa_content_cb (WebKitUserContentManager *manager,
-                                  WebKitJavascriptResult   *result,
+                                  JSCValue                 *value,
                                   EphyFirefoxSyncDialog    *sync_dialog)
 {
   JsonObject *data = NULL;
@@ -336,7 +369,7 @@ sync_message_from_fxa_content_cb (WebKitUserContentManager *manager,
   char *error_msg = NULL;
   gboolean is_error = FALSE;
 
-  message = jsc_value_to_string (webkit_javascript_result_get_js_value (result));
+  message = jsc_value_to_string (value);
   if (!message) {
     g_warning ("Failed to get JavaScript result as string");
     is_error = TRUE;
@@ -400,16 +433,16 @@ out:
 
 static void
 sync_open_webmail_clicked_cb (WebKitUserContentManager *manager,
-                              WebKitJavascriptResult   *result,
+                              JSCValue                 *value,
                               EphyFirefoxSyncDialog    *sync_page)
 {
   EphyShell *shell;
   EphyEmbed *embed;
   GtkWindow *window;
-  GtkWidget *prefs_dialog;
+  GtkRoot *prefs_dialog;
   char *url;
 
-  url = jsc_value_to_string (webkit_javascript_result_get_js_value (result));
+  url = jsc_value_to_string (value);
   if (url) {
     /* Open a new tab to the webmail URL. */
     shell = ephy_shell_get_default ();
@@ -419,8 +452,8 @@ sync_open_webmail_clicked_cb (WebKitUserContentManager *manager,
     ephy_web_view_load_url (ephy_embed_get_web_view (embed), url);
 
     /* Close the preferences dialog. */
-    prefs_dialog = gtk_widget_get_toplevel (GTK_WIDGET (sync_page));
-    gtk_widget_destroy (GTK_WIDGET (prefs_dialog));
+    prefs_dialog = gtk_widget_get_root (GTK_WIDGET (sync_page));
+    gtk_window_destroy (GTK_WINDOW (prefs_dialog));
 
     g_free (url);
   }
@@ -430,10 +463,9 @@ static void
 sync_setup_firefox_iframe (EphyFirefoxSyncDialog *sync_dialog)
 {
   EphyEmbedShell *shell;
-  WebKitWebsiteDataManager *manager;
+  WebKitNetworkSession *network_session;
   WebKitWebContext *embed_context;
   WebKitWebContext *sync_context;
-  GtkWidget *frame;
   const char *script;
 
   if (!sync_dialog->fxa_web_view) {
@@ -468,14 +500,16 @@ sync_setup_firefox_iframe (EphyFirefoxSyncDialog *sync_dialog)
                       G_CALLBACK (sync_open_webmail_clicked_cb),
                       sync_dialog);
     webkit_user_content_manager_register_script_message_handler (sync_dialog->fxa_manager,
-                                                                 "toChromeMessageHandler");
+                                                                 "toChromeMessageHandler",
+                                                                 NULL);
     webkit_user_content_manager_register_script_message_handler (sync_dialog->fxa_manager,
-                                                                 "openWebmailClickHandler");
+                                                                 "openWebmailClickHandler",
+                                                                 NULL);
 
     shell = ephy_embed_shell_get_default ();
     embed_context = ephy_embed_shell_get_web_context (shell);
-    manager = webkit_web_context_get_website_data_manager (embed_context);
-    sync_context = webkit_web_context_new_with_website_data_manager (manager);
+    network_session = ephy_embed_shell_get_network_session (shell);
+    sync_context = webkit_web_context_new ();
     webkit_web_context_set_preferred_languages (sync_context,
                                                 g_object_get_data (G_OBJECT (embed_context), "preferred-languages"));
 
@@ -483,16 +517,14 @@ sync_setup_firefox_iframe (EphyFirefoxSyncDialog *sync_dialog)
                                                                "user-content-manager", sync_dialog->fxa_manager,
                                                                "settings", ephy_embed_prefs_get_settings (),
                                                                "web-context", sync_context,
+                                                               "network-session", network_session,
                                                                NULL));
+    gtk_widget_set_overflow (GTK_WIDGET (sync_dialog->fxa_web_view), GTK_OVERFLOW_HIDDEN);
+    gtk_widget_add_css_class (GTK_WIDGET (sync_dialog->fxa_web_view), "card");
     gtk_widget_set_vexpand (GTK_WIDGET (sync_dialog->fxa_web_view), TRUE);
     gtk_widget_set_visible (GTK_WIDGET (sync_dialog->fxa_web_view), TRUE);
-    frame = gtk_frame_new (NULL);
-    gtk_widget_set_visible (frame, TRUE);
-    gtk_container_add (GTK_CONTAINER (frame),
-                       GTK_WIDGET (sync_dialog->fxa_web_view));
-    gtk_box_pack_start (GTK_BOX (sync_dialog->sync_firefox_iframe_box),
-                        frame,
-                        FALSE, TRUE, 0);
+    gtk_box_append (GTK_BOX (sync_dialog->sync_firefox_iframe_box),
+                    GTK_WIDGET (sync_dialog->fxa_web_view));
 
     g_object_unref (sync_context);
   }
@@ -509,12 +541,13 @@ on_sync_sign_out_button_clicked (GtkWidget             *button,
 
   ephy_sync_service_sign_out (service);
 
-  /* Show Firefox Accounts iframe. */
+  /* Show Mozilla Accounts iframe. */
   sync_setup_firefox_iframe (sync_dialog);
-  gtk_widget_hide (sync_dialog->sync_firefox_account_group);
-  gtk_widget_hide (sync_dialog->sync_options_group);
-  gtk_widget_show (sync_dialog->sync_page_group);
-  hdy_action_row_set_subtitle (HDY_ACTION_ROW (sync_dialog->sync_firefox_account_row), NULL);
+  gtk_widget_set_visible (sync_dialog->sync_now_button, FALSE);
+  gtk_widget_set_visible (sync_dialog->sync_firefox_account_group, FALSE);
+  gtk_widget_set_visible (sync_dialog->sync_options_group, FALSE);
+  gtk_widget_set_visible (sync_dialog->sync_page_group, TRUE);
+  adw_action_row_set_subtitle (ADW_ACTION_ROW (sync_dialog->sync_firefox_account_row), NULL);
 }
 
 static void
@@ -538,7 +571,7 @@ on_sync_synced_tabs_button_clicked (GtkWidget             *button,
   synced_tabs_dialog = synced_tabs_dialog_new (manager);
   gtk_window_set_transient_for (GTK_WINDOW (synced_tabs_dialog), GTK_WINDOW (sync_dialog));
   gtk_window_set_modal (GTK_WINDOW (synced_tabs_dialog), TRUE);
-  gtk_window_present_with_time (GTK_WINDOW (synced_tabs_dialog), gtk_get_current_event_time ());
+  gtk_window_present (GTK_WINDOW (synced_tabs_dialog));
 }
 
 static void
@@ -558,10 +591,10 @@ on_sync_device_name_save_button_clicked (GtkWidget             *button,
   EphySyncService *service = ephy_shell_get_sync_service (ephy_shell_get_default ());
   const char *text;
 
-  text = gtk_entry_get_text (GTK_ENTRY (sync_dialog->sync_device_name_entry));
+  text = gtk_editable_get_text (GTK_EDITABLE (sync_dialog->sync_device_name_entry));
   if (!g_strcmp0 (text, "")) {
     char *name = ephy_sync_utils_get_device_name ();
-    gtk_entry_set_text (GTK_ENTRY (sync_dialog->sync_device_name_entry), name);
+    gtk_editable_set_text (GTK_EDITABLE (sync_dialog->sync_device_name_entry), name);
     g_free (name);
   } else {
     ephy_sync_service_update_device_name (service, text);
@@ -580,7 +613,7 @@ on_sync_device_name_cancel_button_clicked (GtkWidget             *button,
   char *name;
 
   name = ephy_sync_utils_get_device_name ();
-  gtk_entry_set_text (GTK_ENTRY (sync_dialog->sync_device_name_entry), name);
+  gtk_editable_set_text (GTK_EDITABLE (sync_dialog->sync_device_name_entry), name);
 
   gtk_widget_set_sensitive (GTK_WIDGET (sync_dialog->sync_device_name_entry), FALSE);
   gtk_widget_set_visible (GTK_WIDGET (sync_dialog->sync_device_name_change_button), TRUE);
@@ -590,6 +623,15 @@ on_sync_device_name_cancel_button_clicked (GtkWidget             *button,
   g_free (name);
 }
 
+static gchar *
+get_sync_frequency_minutes_name (EphySyncFrequency *value)
+{
+  guint n_minutes = value->frequency;
+  const char *minutes_text = ngettext ("%u min", "%u mins", n_minutes);
+
+  return g_strdup_printf (minutes_text, n_minutes);
+}
+
 static void
 prefs_sync_page_finalize (GObject *object)
 {
@@ -597,9 +639,11 @@ prefs_sync_page_finalize (GObject *object)
 
   if (sync_dialog->fxa_web_view != NULL) {
     webkit_user_content_manager_unregister_script_message_handler (sync_dialog->fxa_manager,
-                                                                   "toChromeMessageHandler");
+                                                                   "toChromeMessageHandler",
+                                                                   NULL);
     webkit_user_content_manager_unregister_script_message_handler (sync_dialog->fxa_manager,
-                                                                   "openWebmailClickHandler");
+                                                                   "openWebmailClickHandler",
+                                                                   NULL);
     webkit_user_script_unref (sync_dialog->fxa_script);
     g_object_unref (sync_dialog->fxa_manager);
   }
@@ -624,10 +668,10 @@ ephy_firefox_sync_dialog_class_init (EphyFirefoxSyncDialogClass *klass)
   gtk_widget_class_bind_template_child (widget_class, EphyFirefoxSyncDialog, sync_firefox_account_group);
   gtk_widget_class_bind_template_child (widget_class, EphyFirefoxSyncDialog, sync_firefox_account_row);
   gtk_widget_class_bind_template_child (widget_class, EphyFirefoxSyncDialog, sync_options_group);
-  gtk_widget_class_bind_template_child (widget_class, EphyFirefoxSyncDialog, sync_bookmarks_switch);
-  gtk_widget_class_bind_template_child (widget_class, EphyFirefoxSyncDialog, sync_passwords_switch);
-  gtk_widget_class_bind_template_child (widget_class, EphyFirefoxSyncDialog, sync_history_switch);
-  gtk_widget_class_bind_template_child (widget_class, EphyFirefoxSyncDialog, sync_open_tabs_switch);
+  gtk_widget_class_bind_template_child (widget_class, EphyFirefoxSyncDialog, sync_bookmarks_row);
+  gtk_widget_class_bind_template_child (widget_class, EphyFirefoxSyncDialog, sync_passwords_row);
+  gtk_widget_class_bind_template_child (widget_class, EphyFirefoxSyncDialog, sync_history_row);
+  gtk_widget_class_bind_template_child (widget_class, EphyFirefoxSyncDialog, sync_open_tabs_row);
   gtk_widget_class_bind_template_child (widget_class, EphyFirefoxSyncDialog, sync_frequency_row);
   gtk_widget_class_bind_template_child (widget_class, EphyFirefoxSyncDialog, sync_now_button);
   gtk_widget_class_bind_template_child (widget_class, EphyFirefoxSyncDialog, synced_tabs_button);
@@ -642,6 +686,9 @@ ephy_firefox_sync_dialog_class_init (EphyFirefoxSyncDialogClass *klass)
   gtk_widget_class_bind_template_callback (widget_class, on_sync_device_name_change_button_clicked);
   gtk_widget_class_bind_template_callback (widget_class, on_sync_device_name_save_button_clicked);
   gtk_widget_class_bind_template_callback (widget_class, on_sync_device_name_cancel_button_clicked);
+  gtk_widget_class_bind_template_callback (widget_class, get_sync_frequency_minutes_name);
+
+  gtk_widget_class_add_binding_action (widget_class, GDK_KEY_Escape, 0, "window.close", NULL);
 }
 
 static gboolean
@@ -651,11 +698,11 @@ sync_frequency_get_mapping (GValue   *value,
 {
   uint minutes = g_variant_get_uint32 (variant);
 
-  for (gint i = 0; i < (gint)G_N_ELEMENTS (sync_frequency_minutes); i++) {
+  for (guint i = 0; i < (guint)G_N_ELEMENTS (sync_frequency_minutes); i++) {
     if (sync_frequency_minutes[i] != minutes)
       continue;
 
-    g_value_set_int (value, i);
+    g_value_set_uint (value, i);
 
     return TRUE;
   }
@@ -668,9 +715,9 @@ sync_frequency_set_mapping (const GValue       *value,
                             const GVariantType *expected_type,
                             gpointer            user_data)
 {
-  gint i = g_value_get_int (value);
+  guint i = g_value_get_uint (value);
 
-  if (i >= (gint)G_N_ELEMENTS (sync_frequency_minutes))
+  if (i >= (guint)G_N_ELEMENTS (sync_frequency_minutes))
     return NULL;
 
   return g_variant_new_uint32 (sync_frequency_minutes[i]);
@@ -679,29 +726,19 @@ sync_frequency_set_mapping (const GValue       *value,
 static GListModel *
 create_sync_frequency_minutes_model ()
 {
-  GListStore *list_store = g_list_store_new (HDY_TYPE_VALUE_OBJECT);
-  HdyValueObject *obj;
-  g_auto (GValue) value = G_VALUE_INIT;
+  GListStore *list_store = g_list_store_new (EPHY_TYPE_SYNC_FREQUENCY);
   guint i;
 
-  g_value_init (&value, G_TYPE_UINT);
-
   for (i = 0; i < G_N_ELEMENTS (sync_frequency_minutes); i++) {
-    g_value_set_uint (&value, sync_frequency_minutes[i]);
-    obj = hdy_value_object_new (&value);
-    g_list_store_insert (list_store, i, obj);
-    g_clear_object (&obj);
+    g_autoptr (EphySyncFrequency) frequency =
+      ephy_sync_frequency_new (sync_frequency_minutes[i]);
+
+    g_list_store_insert (list_store, i, frequency);
   }
 
   return G_LIST_MODEL (list_store);
 }
 
-static gchar *
-get_sync_frequency_minutes_name (HdyValueObject *value)
-{
-  return g_strdup_printf ("%u min", g_value_get_uint (hdy_value_object_get_value (value)));
-}
-
 void
 ephy_firefox_sync_dialog_setup (EphyFirefoxSyncDialog *sync_dialog)
 {
@@ -711,54 +748,52 @@ ephy_firefox_sync_dialog_setup (EphyFirefoxSyncDialog *sync_dialog)
   char *name = ephy_sync_utils_get_device_name ();
   g_autoptr (GListModel) sync_frequency_minutes_model = create_sync_frequency_minutes_model ();
 
-  gtk_entry_set_text (GTK_ENTRY (sync_dialog->sync_device_name_entry), name);
+  gtk_editable_set_text (GTK_EDITABLE (sync_dialog->sync_device_name_entry), name);
 
   if (!user) {
     sync_setup_firefox_iframe (sync_dialog);
-    gtk_widget_hide (sync_dialog->sync_firefox_account_group);
-    gtk_widget_hide (sync_dialog->sync_options_group);
+    gtk_widget_set_visible (sync_dialog->sync_now_button, FALSE);
+    gtk_widget_set_visible (sync_dialog->sync_firefox_account_group, FALSE);
+    gtk_widget_set_visible (sync_dialog->sync_options_group, FALSE);
   } else {
     sync_set_last_sync_time (sync_dialog);
-    hdy_preferences_row_set_title (HDY_PREFERENCES_ROW (sync_dialog->sync_firefox_account_row), user);
-    gtk_widget_hide (sync_dialog->sync_page_group);
+    adw_preferences_row_set_title (ADW_PREFERENCES_ROW (sync_dialog->sync_firefox_account_row), user);
+    gtk_widget_set_visible (sync_dialog->sync_page_group, FALSE);
   }
 
   g_settings_bind (sync_settings,
                    EPHY_PREFS_SYNC_BOOKMARKS_ENABLED,
-                   sync_dialog->sync_bookmarks_switch,
+                   sync_dialog->sync_bookmarks_row,
                    "active",
                    G_SETTINGS_BIND_DEFAULT);
   g_settings_bind (sync_settings,
                    EPHY_PREFS_SYNC_PASSWORDS_ENABLED,
-                   sync_dialog->sync_passwords_switch,
+                   sync_dialog->sync_passwords_row,
                    "active",
                    G_SETTINGS_BIND_DEFAULT);
   g_settings_bind (sync_settings,
                    EPHY_PREFS_SYNC_HISTORY_ENABLED,
-                   sync_dialog->sync_history_switch,
+                   sync_dialog->sync_history_row,
                    "active",
                    G_SETTINGS_BIND_DEFAULT);
   g_settings_bind (sync_settings,
                    EPHY_PREFS_SYNC_OPEN_TABS_ENABLED,
-                   sync_dialog->sync_open_tabs_switch,
+                   sync_dialog->sync_open_tabs_row,
                    "active",
                    G_SETTINGS_BIND_DEFAULT);
 
-  hdy_combo_row_bind_name_model (HDY_COMBO_ROW (sync_dialog->sync_frequency_row),
-                                 sync_frequency_minutes_model,
-                                 (HdyComboRowGetNameFunc)get_sync_frequency_minutes_name,
-                                 NULL,
-                                 NULL);
+  adw_combo_row_set_model (ADW_COMBO_ROW (sync_dialog->sync_frequency_row),
+                           sync_frequency_minutes_model);
   g_settings_bind_with_mapping (sync_settings,
                                 EPHY_PREFS_SYNC_FREQUENCY,
                                 sync_dialog->sync_frequency_row,
-                                "selected-index",
+                                "selected",
                                 G_SETTINGS_BIND_DEFAULT,
                                 sync_frequency_get_mapping,
                                 sync_frequency_set_mapping,
                                 NULL, NULL);
 
-  g_object_bind_property (sync_dialog->sync_open_tabs_switch, "active",
+  g_object_bind_property (sync_dialog->sync_open_tabs_row, "active",
                           sync_dialog->synced_tabs_button, "sensitive",
                           G_BINDING_SYNC_CREATE);
 
@@ -771,16 +806,16 @@ ephy_firefox_sync_dialog_setup (EphyFirefoxSyncDialog *sync_dialog)
   g_signal_connect_object (service, "sync-finished",
                            G_CALLBACK (sync_finished_cb),
                            sync_dialog, 0);
-  g_signal_connect_object (sync_dialog->sync_bookmarks_switch, "notify::active",
+  g_signal_connect_object (sync_dialog->sync_bookmarks_row, "notify::active",
                            G_CALLBACK (sync_collection_toggled_cb),
                            sync_dialog, 0);
-  g_signal_connect_object (sync_dialog->sync_passwords_switch, "notify::active",
+  g_signal_connect_object (sync_dialog->sync_passwords_row, "notify::active",
                            G_CALLBACK (sync_collection_toggled_cb),
                            sync_dialog, 0);
-  g_signal_connect_object (sync_dialog->sync_history_switch, "notify::active",
+  g_signal_connect_object (sync_dialog->sync_history_row, "notify::active",
                            G_CALLBACK (sync_collection_toggled_cb),
                            sync_dialog, 0);
-  g_signal_connect_object (sync_dialog->sync_open_tabs_switch, "notify::active",
+  g_signal_connect_object (sync_dialog->sync_open_tabs_row, "notify::active",
                            G_CALLBACK (sync_collection_toggled_cb),
                            sync_dialog, 0);
 
