@@ -31,9 +31,7 @@
 #include "ephy-suggestion-model.h"
 #include "ephy-title-widget.h"
 #include "ephy-uri-helpers.h"
-#include "ephy-widgets-type-builtins.h"
 
-#include <dazzle.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 #include <string.h>
@@ -50,7 +48,7 @@ struct _EphyLocationController {
 
   EphyWindow *window;
   EphyTitleWidget *title_widget;
-  GtkGesture *longpress_gesture;
+  GtkEventController *focus_controller;
   char *address;
   gboolean editable;
   gboolean sync_address_is_blocked;
@@ -58,8 +56,6 @@ struct _EphyLocationController {
 };
 
 static void ephy_location_controller_finalize (GObject *object);
-static void user_changed_cb (GtkWidget              *widget,
-                             EphyLocationController *controller);
 static void sync_address (EphyLocationController *controller,
                           GParamSpec             *pspec,
                           GtkWidget              *widget);
@@ -74,9 +70,64 @@ enum {
 };
 static GParamSpec *obj_properties[LAST_PROP];
 
-G_DEFINE_TYPE_WITH_CODE (EphyLocationController, ephy_location_controller, G_TYPE_OBJECT,
-                         G_IMPLEMENT_INTERFACE (EPHY_TYPE_LINK,
-                                                NULL))
+G_DEFINE_FINAL_TYPE_WITH_CODE (EphyLocationController, ephy_location_controller, G_TYPE_OBJECT,
+                               G_IMPLEMENT_INTERFACE (EPHY_TYPE_LINK,
+                                                      NULL))
+
+static gboolean
+handle_ephy_tab_uri (EphyLocationController *controller,
+                     const char             *content)
+{
+  EphyTabView *tab_view = ephy_window_get_tab_view (controller->window);
+  GtkWidget *tab;
+  EphyWebView *webview;
+  int window_id;
+  int tab_id;
+  g_auto (GStrv) split = g_strsplit (content + strlen ("ephy-tab://"), "@", -1);
+
+  if (g_strv_length (split) != 2)
+    return FALSE;
+
+  window_id = atoi (split[1]);
+  tab_id = atoi (split[0]);
+  tab = ephy_tab_view_get_selected_page (tab_view);
+  webview = ephy_embed_get_web_view (EPHY_EMBED (tab));
+
+  if (window_id != 0) {
+    GApplication *application;
+    EphyEmbedShell *shell;
+    EphyWindow *window;
+    GList *windows;
+
+    shell = ephy_embed_shell_get_default ();
+    application = G_APPLICATION (shell);
+    windows = gtk_application_get_windows (GTK_APPLICATION (application));
+
+    if ((guint)window_id >= g_list_length (windows))
+      return FALSE;
+
+    window = g_list_nth_data (windows, window_id);
+    tab_view = ephy_window_get_tab_view (window);
+
+    /* FIXME: this doesn't actually work.
+     * https://gitlab.gnome.org/GNOME/epiphany/-/issues/1908
+     */
+    gtk_window_present (GTK_WINDOW (window));
+  }
+
+  if (tab_id >= ephy_tab_view_get_n_pages (tab_view))
+    return FALSE;
+
+  ephy_tab_view_select_nth_page (tab_view, tab_id);
+
+  if (ephy_web_view_is_overview (webview)) {
+    if (window_id != 0)
+      tab_view = ephy_window_get_tab_view (controller->window);
+    ephy_tab_view_close (tab_view, tab);
+  }
+
+  return TRUE;
+}
 
 static void
 entry_activate_cb (EphyLocationEntry      *entry,
@@ -86,52 +137,18 @@ entry_activate_cb (EphyLocationEntry      *entry,
   const char *content;
   char *address;
   char *effective_address;
-  GtkWidget *inner_entry;
 
   if (controller->sync_address_is_blocked) {
     controller->sync_address_is_blocked = FALSE;
     g_signal_handlers_unblock_by_func (controller, G_CALLBACK (sync_address), entry);
   }
 
-  inner_entry = ephy_location_entry_get_entry (entry);
-  content = gtk_entry_get_text (GTK_ENTRY (inner_entry));
+  content = gtk_editable_get_text (GTK_EDITABLE (entry));
   if (content == NULL || content[0] == '\0')
     return;
 
-  if (g_str_has_prefix (content, "ephy-tab://")) {
-    EphyTabView *tab_view = ephy_window_get_tab_view (controller->window);
-    GtkWidget *tab;
-    EphyWebView *webview;
-    g_auto (GStrv) split = g_strsplit (content + strlen ("ephy-tab://"), "@", -1);
-
-    g_assert (g_strv_length (split) == 2);
-
-    tab = ephy_tab_view_get_selected_page (tab_view);
-    webview = ephy_embed_get_web_view (EPHY_EMBED (tab));
-
-    if (atoi (split[1]) != 0) {
-      GApplication *application;
-      EphyEmbedShell *shell;
-      EphyWindow *window;
-      GList *windows;
-
-      shell = ephy_embed_shell_get_default ();
-      application = G_APPLICATION (shell);
-      windows = gtk_application_get_windows (GTK_APPLICATION (application));
-
-      window = g_list_nth_data (windows, atoi (split[1]));
-      tab_view = ephy_window_get_tab_view (window);
-
-      gtk_window_present (GTK_WINDOW (window));
-    }
-
-    ephy_tab_view_select_nth_page (tab_view, atoi (split[0]));
-
-    if (ephy_web_view_is_overview (webview))
-      ephy_tab_view_close (tab_view, tab);
-
+  if (g_str_has_prefix (content, "ephy-tab://") && handle_ephy_tab_uri (controller, content))
     return;
-  }
 
   address = g_strdup (content);
   effective_address = ephy_embed_utils_normalize_or_autosearch_address (g_strstrip (address));
@@ -141,7 +158,7 @@ entry_activate_cb (EphyLocationEntry      *entry,
     /* After normalization there are still some cases that are
      * impossible to tell apart. One example is <URI>:<PORT> and <NON
      * WEB SCHEME>:<DATA>. To fix this, let's do a HEAD request to the
-     * effective URI prefxed with http://; if we get OK Status the URI
+     * effective URI prefixed with http://; if we get OK Status the URI
      * exists, and we'll go ahead, otherwise we'll try to launch a
      * proper handler through gtk_show_uri. We only do this in
      * ephy_web_view_load_url, since this case is only relevant for URIs
@@ -162,17 +179,14 @@ entry_activate_cb (EphyLocationEntry      *entry,
 
 static void
 user_changed_cb (GtkWidget              *widget,
+                 const char             *address,
                  EphyLocationController *controller)
 {
-  const char *address;
-  DzlSuggestionEntry *entry = DZL_SUGGESTION_ENTRY (ephy_location_entry_get_entry (EPHY_LOCATION_ENTRY (widget)));
   GListModel *model;
 
-  address = dzl_suggestion_entry_get_typed_text (entry);
-
   LOG ("user_changed_cb, address %s", address);
 
-  model = dzl_suggestion_entry_get_model (entry);
+  model = ephy_location_entry_get_model (EPHY_LOCATION_ENTRY (controller->title_widget));
 
   ephy_suggestion_model_query_async (EPHY_SUGGESTION_MODEL (model), address, TRUE, NULL, NULL, NULL);
 }
@@ -214,10 +228,8 @@ get_title_cb (EphyLocationEntry      *entry,
   return g_strdup (ephy_embed_get_title (embed));
 }
 
-static gboolean
-focus_in_event_cb (GtkWidget              *entry,
-                   GdkEventFocus          *event,
-                   EphyLocationController *controller)
+static void
+focus_enter_cb (EphyLocationController *controller)
 {
   const char *address;
 
@@ -226,23 +238,17 @@ focus_in_event_cb (GtkWidget              *entry,
   address = ephy_title_widget_get_address (controller->title_widget);
   if (!controller->sync_address_is_blocked && address && *address) {
     controller->sync_address_is_blocked = TRUE;
-    g_signal_handlers_block_by_func (controller, G_CALLBACK (sync_address), entry);
+    g_signal_handlers_block_by_func (controller, G_CALLBACK (sync_address), controller->title_widget);
   }
-
-  return FALSE;
 }
 
-static gboolean
-focus_out_event_cb (GtkWidget              *entry,
-                    GdkEventFocus          *event,
-                    EphyLocationController *controller)
+static void
+focus_leave_cb (EphyLocationController *controller)
 {
   if (controller->sync_address_is_blocked) {
     controller->sync_address_is_blocked = FALSE;
-    g_signal_handlers_unblock_by_func (controller, G_CALLBACK (sync_address), entry);
+    g_signal_handlers_unblock_by_func (controller, G_CALLBACK (sync_address), controller->title_widget);
   }
-
-  return FALSE;
 }
 
 static void
@@ -254,17 +260,6 @@ notify_selected_index_cb (EphyLocationController *controller)
   }
 }
 
-static void
-longpress_gesture_cb (GtkGestureLongPress *gesture,
-                      gdouble              x,
-                      gdouble              y,
-                      gpointer             user_data)
-{
-  GtkWidget *entry = user_data;
-
-  gtk_editable_select_region (GTK_EDITABLE (entry), 0, -1);
-}
-
 static void
 reader_mode_changed_cb (EphyLocationEntry *lentry,
                         gboolean           active,
@@ -286,7 +281,8 @@ ephy_location_controller_constructed (GObject *object)
   EphyBookmarksManager *bookmarks_manager;
   EphySuggestionModel *model;
   EphyTabView *tab_view;
-  GtkWidget *widget, *entry;
+  GtkWidget *widget;
+  GtkEventController *focus_controller;
 
   G_OBJECT_CLASS (ephy_location_controller_parent_class)->constructed (object);
 
@@ -304,24 +300,19 @@ ephy_location_controller_constructed (GObject *object)
   if (!EPHY_IS_LOCATION_ENTRY (controller->title_widget))
     return;
 
-  entry = ephy_location_entry_get_entry (EPHY_LOCATION_ENTRY (controller->title_widget));
   g_signal_connect (controller->title_widget, "user-changed", G_CALLBACK (user_changed_cb), controller);
 
-  controller->longpress_gesture = gtk_gesture_long_press_new (entry);
-  gtk_gesture_single_set_touch_only (GTK_GESTURE_SINGLE (controller->longpress_gesture), TRUE);
-  g_signal_connect (controller->longpress_gesture, "pressed", G_CALLBACK (longpress_gesture_cb), entry);
-
   history_service = ephy_embed_shell_get_global_history_service (ephy_embed_shell_get_default ());
   bookmarks_manager = ephy_shell_get_bookmarks_manager (ephy_shell_get_default ());
   model = ephy_suggestion_model_new (history_service, bookmarks_manager);
-  dzl_suggestion_entry_set_model (DZL_SUGGESTION_ENTRY (entry), G_LIST_MODEL (model));
+  ephy_location_entry_set_model (EPHY_LOCATION_ENTRY (controller->title_widget), G_LIST_MODEL (model));
   g_object_unref (model);
 
   g_signal_connect (controller->title_widget, "reader-mode-changed",
                     G_CALLBACK (reader_mode_changed_cb), controller);
 
   g_object_bind_property (controller, "editable",
-                          entry, "editable",
+                          widget, "editable",
                           G_BINDING_SYNC_CREATE);
 
   g_signal_connect_object (widget, "activate",
@@ -331,10 +322,13 @@ ephy_location_controller_constructed (GObject *object)
                            G_CALLBACK (get_location_cb), controller, 0);
   g_signal_connect_object (widget, "get-title",
                            G_CALLBACK (get_title_cb), controller, 0);
-  g_signal_connect_object (widget, "focus-in-event",
-                           G_CALLBACK (focus_in_event_cb), controller, 0);
-  g_signal_connect_object (widget, "focus-out-event",
-                           G_CALLBACK (focus_out_event_cb), controller, 0);
+
+  focus_controller = gtk_event_controller_focus_new ();
+  g_signal_connect_object (focus_controller, "enter",
+                           G_CALLBACK (focus_enter_cb), controller, G_CONNECT_SWAPPED);
+  g_signal_connect_object (focus_controller, "leave",
+                           G_CALLBACK (focus_leave_cb), controller, G_CONNECT_SWAPPED);
+  gtk_widget_add_controller (widget, focus_controller);
 }
 
 static void
@@ -391,13 +385,15 @@ ephy_location_controller_dispose (GObject *object)
   if (!controller->title_widget)
     return;
 
-  g_clear_object (&controller->longpress_gesture);
-
   if (EPHY_IS_LOCATION_ENTRY (controller->title_widget)) {
     g_signal_handlers_disconnect_matched (controller, G_SIGNAL_MATCH_DATA,
                                           0, 0, NULL, NULL, controller->title_widget);
     g_signal_handlers_disconnect_matched (controller->title_widget, G_SIGNAL_MATCH_DATA,
                                           0, 0, NULL, NULL, controller);
+
+    gtk_widget_remove_controller (GTK_WIDGET (controller->title_widget),
+                                  controller->focus_controller);
+    controller->focus_controller = NULL;
   }
   controller->title_widget = NULL;
 
@@ -422,8 +418,7 @@ ephy_location_controller_class_init (EphyLocationControllerClass *class)
    */
   obj_properties[PROP_ADDRESS] =
     g_param_spec_string ("address",
-                         "Address",
-                         "The address of the current location",
+                         NULL, NULL,
                          "",
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
@@ -434,8 +429,7 @@ ephy_location_controller_class_init (EphyLocationControllerClass *class)
    */
   obj_properties[PROP_EDITABLE] =
     g_param_spec_boolean ("editable",
-                          "Editable",
-                          "Whether the location bar entry can be edited",
+                          NULL, NULL,
                           TRUE,
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
@@ -446,8 +440,7 @@ ephy_location_controller_class_init (EphyLocationControllerClass *class)
    */
   obj_properties[PROP_WINDOW] =
     g_param_spec_object ("window",
-                         "Window",
-                         "The parent window",
+                         NULL, NULL,
                          G_TYPE_OBJECT,
                          G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS | G_PARAM_CONSTRUCT_ONLY);
 
@@ -458,8 +451,7 @@ ephy_location_controller_class_init (EphyLocationControllerClass *class)
    */
   obj_properties[PROP_TITLE_WIDGET] =
     g_param_spec_object ("title-widget",
-                         "Title widget",
-                         "The title widget whose address will be managed",
+                         NULL, NULL,
                          G_TYPE_OBJECT,
                          G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS | G_PARAM_CONSTRUCT_ONLY);
 
