@@ -30,7 +30,6 @@
 #include "ephy-embed-utils.h"
 #include "ephy-embed.h"
 #include "ephy-file-helpers.h"
-#include "ephy-gui.h"
 #include "ephy-link.h"
 #include "ephy-tab-view.h"
 #include "ephy-prefs.h"
@@ -38,7 +37,6 @@
 #include "ephy-shell.h"
 #include "ephy-string.h"
 #include "ephy-tab-view.h"
-#include "ephy-view-source-handler.h"
 #include "ephy-window.h"
 
 #include <glib/gi18n.h>
@@ -47,7 +45,7 @@
 #include <libxml/xmlwriter.h>
 
 typedef struct {
-  EphyTabView *tab_view;
+  EphyTabView *tab_view; /* nullable, weak ref */
   gint ref_count;
 } TabViewTracker;
 
@@ -80,7 +78,7 @@ static GParamSpec *obj_properties[LAST_PROP];
 
 static void ephy_session_save_now (EphySession *session);
 
-G_DEFINE_TYPE (EphySession, ephy_session, G_TYPE_OBJECT)
+G_DEFINE_FINAL_TYPE (EphySession, ephy_session, G_TYPE_OBJECT)
 
 /* Helper functions */
 
@@ -153,6 +151,8 @@ tab_view_tracker_new (EphyTabView *tab_view)
 {
   TabViewTracker *tracker = g_new0 (TabViewTracker, 1);
 
+  g_assert (!tab_view || EPHY_IS_TAB_VIEW (tab_view));
+
   tracker->ref_count = 1;
   tab_view_tracker_set_tab_view (tracker, tab_view);
 
@@ -256,7 +256,7 @@ ephy_session_undo_close_tab (EphySession *session)
       flags |= EPHY_NEW_TAB_FIRST;
     }
 
-    window = EPHY_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (tab_view)));
+    window = EPHY_WINDOW (gtk_widget_get_root (GTK_WIDGET (tab_view)));
     new_tab = ephy_shell_new_tab (ephy_shell_get_default (),
                                   window, embed,
                                   flags);
@@ -326,25 +326,32 @@ ephy_session_get_can_undo_tab_closed (EphySession *session)
 }
 
 static void
-tab_view_page_attached_cb (HdyTabView  *tab_view,
-                           HdyTabPage  *page,
+tab_view_page_attached_cb (AdwTabView  *tab_view,
+                           AdwTabPage  *page,
                            guint        position,
                            EphySession *session)
 {
-  EphyEmbed *embed = EPHY_EMBED (hdy_tab_page_get_child (page));
+  EphyEmbed *embed = EPHY_EMBED (adw_tab_page_get_child (page));
 
   g_signal_connect (ephy_embed_get_web_view (embed), "load-changed",
                     G_CALLBACK (load_changed_cb), session);
 }
 
 static void
-tab_view_page_detached_cb (HdyTabView  *tab_view,
-                           HdyTabPage  *page,
+tab_view_page_detached_cb (AdwTabView  *tab_view,
+                           AdwTabPage  *page,
                            gint         position,
                            EphySession *session)
 {
-  EphyEmbed *embed = EPHY_EMBED (hdy_tab_page_get_child (page));
-  EphyTabView *ephy_tab_view = EPHY_TAB_VIEW (g_object_get_data (G_OBJECT (tab_view), "ephy-tab-view"));
+  EphyEmbed *embed = EPHY_EMBED (adw_tab_page_get_child (page));
+  EphyTabView *ephy_tab_view;
+
+  g_assert (ADW_IS_TAB_VIEW (tab_view));
+  /* The EphyTabView will be NULL if closing the last tab of a window, which
+   * indicates the window is being destroyed. This is OK.
+   */
+  ephy_tab_view = EPHY_GET_TAB_VIEW_FROM_ADW_TAB_VIEW (tab_view);
+  g_assert (!ephy_tab_view || EPHY_IS_TAB_VIEW (ephy_tab_view));
 
   ephy_session_save (session);
 
@@ -356,8 +363,8 @@ tab_view_page_detached_cb (HdyTabView  *tab_view,
 }
 
 static void
-tab_view_page_reordered_cb (HdyTabView  *tab_view,
-                            HdyTabPage  *page,
+tab_view_page_reordered_cb (AdwTabView  *tab_view,
+                            AdwTabPage  *page,
                             guint        position,
                             EphySession *session)
 {
@@ -365,7 +372,7 @@ tab_view_page_reordered_cb (HdyTabView  *tab_view,
 }
 
 static void
-tab_view_notify_selected_page_cb (HdyTabView  *tab_view,
+tab_view_notify_selected_page_cb (AdwTabView  *tab_view,
                                   GParamSpec  *pspec,
                                   EphySession *session)
 {
@@ -392,7 +399,7 @@ window_added_cb (GtkApplication *application,
                  EphySession    *session)
 {
   EphyWindow *ephy_window;
-  HdyTabView *tab_view;
+  AdwTabView *tab_view;
 
   ephy_session_save (session);
 
@@ -483,8 +490,7 @@ ephy_session_class_init (EphySessionClass *class)
 
   obj_properties[PROP_CAN_UNDO_TAB_CLOSED] =
     g_param_spec_boolean ("can-undo-tab-closed",
-                          "Can undo tab close",
-                          "Session can undo a tab closure",
+                          NULL, NULL,
                           FALSE,
                           G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
@@ -516,7 +522,8 @@ ephy_session_close (EphySession *session)
 }
 
 typedef struct {
-  GdkRectangle geometry;
+  int width;
+  int height;
   gboolean is_maximized;
   gboolean is_fullscreen;
 
@@ -528,10 +535,9 @@ static void
 get_window_geometry (EphyWindow    *window,
                      SessionWindow *session_window)
 {
+  gtk_window_get_default_size (GTK_WINDOW (window), &session_window->width, &session_window->height);
   session_window->is_maximized = ephy_window_is_maximized (window);
   session_window->is_fullscreen = ephy_window_is_fullscreen (window);
-
-  ephy_window_get_geometry (window, &session_window->geometry);
 }
 
 typedef struct {
@@ -738,46 +744,6 @@ write_tab (xmlTextWriterPtr  writer,
   return ret;
 }
 
-static int
-write_window_geometry (xmlTextWriterPtr  writer,
-                       GdkRectangle     *geometry,
-                       gboolean          is_maximized,
-                       gboolean          is_fullscreen)
-{
-  int ret;
-
-  /* set window properties */
-  ret = xmlTextWriterWriteFormatAttribute (writer, (const xmlChar *)"x", "%d",
-                                           geometry->x);
-  if (ret < 0)
-    return ret;
-
-  ret = xmlTextWriterWriteFormatAttribute (writer, (const xmlChar *)"y", "%d",
-                                           geometry->y);
-  if (ret < 0)
-    return ret;
-
-  ret = xmlTextWriterWriteFormatAttribute (writer, (const xmlChar *)"width", "%d",
-                                           geometry->width);
-  if (ret < 0)
-    return ret;
-
-  ret = xmlTextWriterWriteFormatAttribute (writer, (const xmlChar *)"height", "%d",
-                                           geometry->height);
-  if (ret < 0)
-    return ret;
-
-  ret = xmlTextWriterWriteFormatAttribute (writer, (const xmlChar *)"is-maximized", "%d",
-                                           is_maximized);
-  if (ret < 0)
-    return ret;
-
-  ret = xmlTextWriterWriteFormatAttribute (writer, (const xmlChar *)"is-fullscreen", "%d",
-                                           is_fullscreen);
-
-  return ret;
-}
-
 static int
 write_ephy_window (xmlTextWriterPtr  writer,
                    SessionWindow    *window)
@@ -807,7 +773,19 @@ write_ephy_window (xmlTextWriterPtr  writer,
   if (ret < 0)
     return ret;
 
-  ret = write_window_geometry (writer, &window->geometry, window->is_maximized, window->is_fullscreen);
+  ret = xmlTextWriterWriteFormatAttribute (writer, (const xmlChar *)"width", "%d", window->width);
+  if (ret < 0)
+    return ret;
+
+  ret = xmlTextWriterWriteFormatAttribute (writer, (const xmlChar *)"height", "%d", window->height);
+  if (ret < 0)
+    return ret;
+
+  ret = xmlTextWriterWriteFormatAttribute (writer, (const xmlChar *)"is-maximized", "%d", window->is_maximized);
+  if (ret < 0)
+    return ret;
+
+  ret = xmlTextWriterWriteFormatAttribute (writer, (const xmlChar *)"is-fullscreen", "%d", window->is_fullscreen);
   if (ret < 0)
     return ret;
 
@@ -889,12 +867,13 @@ session_seems_reasonable (GList *windows)
       if (g_str_has_prefix (url, "about:"))
         continue;
 
-      uri = g_uri_parse (url, G_URI_FLAGS_NONE, NULL);
+      uri = g_uri_parse (url, G_URI_FLAGS_PARSE_RELAXED, NULL);
       if (uri) {
         if (g_uri_get_host (uri) != NULL ||
             strcmp (g_uri_get_scheme (uri), "data") == 0 ||
             strcmp (g_uri_get_scheme (uri), "file") == 0 ||
-            strcmp (g_uri_get_scheme (uri), "ephy-reader") == 0)
+            strcmp (g_uri_get_scheme (uri), "ephy-reader") == 0 ||
+            strcmp (g_uri_get_scheme (uri), "view-source") == 0)
           sane = TRUE;
       }
 
@@ -1086,14 +1065,6 @@ confirm_before_recover (EphyWindow *window,
                                  EPHY_WEB_VIEW_ERROR_PAGE_CRASH, NULL, NULL);
 }
 
-static void
-restore_geometry (GtkWindow    *window,
-                  GdkRectangle *geometry)
-{
-  if (geometry->width > 0 && geometry->height > 0)
-    ephy_window_set_default_size (EPHY_WINDOW (window), geometry->width, geometry->height);
-}
-
 typedef struct {
   EphySession *session;
   guint32 user_time;
@@ -1133,12 +1104,20 @@ session_parser_context_free (SessionParserContext *context)
   g_free (context);
 }
 
+static void
+window_destroyed (GtkWidget  *widget,
+                  GtkWidget **widget_pointer)
+{
+  if (widget_pointer)
+    *widget_pointer = NULL;
+}
+
 static void
 session_parse_window (SessionParserContext  *context,
                       const gchar          **names,
                       const gchar          **values)
 {
-  GdkRectangle geometry = { -1, -1, 0, 0 };
+  int width = 0, height = 0;
   gboolean is_maximized = FALSE;
   gboolean is_fullscreen = FALSE;
   guint i;
@@ -1149,23 +1128,17 @@ session_parse_window (SessionParserContext  *context,
   }
 
   context->window = ephy_window_new ();
-  context->destroy_id = g_signal_connect (context->window, "destroy", G_CALLBACK (gtk_widget_destroyed), &context->window);
+  context->destroy_id = g_signal_connect (context->window, "destroy", G_CALLBACK (window_destroyed), &context->window);
 
   for (i = 0; names[i]; i++) {
     gulong int_value;
 
-    if (strcmp (names[i], "x") == 0) {
+    if (strcmp (names[i], "width") == 0) {
       ephy_string_to_int (values[i], &int_value);
-      geometry.x = int_value;
-    } else if (strcmp (names[i], "y") == 0) {
-      ephy_string_to_int (values[i], &int_value);
-      geometry.y = int_value;
-    } else if (strcmp (names[i], "width") == 0) {
-      ephy_string_to_int (values[i], &int_value);
-      geometry.width = int_value;
+      width = int_value;
     } else if (strcmp (names[i], "height") == 0) {
       ephy_string_to_int (values[i], &int_value);
-      geometry.height = int_value;
+      height = int_value;
     } else if (strcmp (names[i], "is-maximized") == 0) {
       ephy_string_to_int (values[i], &int_value);
       is_maximized = int_value != 0;
@@ -1178,7 +1151,8 @@ session_parse_window (SessionParserContext  *context,
     }
   }
 
-  restore_geometry (GTK_WINDOW (context->window), &geometry);
+  if (width > 0 && height > 0)
+    ephy_window_set_default_size (context->window, width, height);
 
   if (is_maximized)
     gtk_window_maximize (GTK_WINDOW (context->window));
@@ -1195,7 +1169,7 @@ session_parse_embed (SessionParserContext  *context,
                      const gchar          **names,
                      const gchar          **values)
 {
-  HdyTabView *tab_view;
+  AdwTabView *tab_view;
   const char *url = NULL;
   const char *title = NULL;
   const char *history = NULL;
@@ -1262,8 +1236,8 @@ session_parse_embed (SessionParserContext  *context,
                                      context->window, NULL, flags,
                                      0);
 
-    hdy_tab_view_set_page_pinned (tab_view,
-                                  hdy_tab_view_get_page (tab_view, GTK_WIDGET (embed)),
+    adw_tab_view_set_page_pinned (tab_view,
+                                  adw_tab_view_get_page (tab_view, GTK_WIDGET (embed)),
                                   is_pin);
 
     web_view = ephy_embed_get_web_view (embed);
@@ -1360,7 +1334,7 @@ session_end_element (GMarkupParseContext  *ctx,
       active_child = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (context->window));
       gtk_widget_grab_focus (GTK_WIDGET (active_child));
       ephy_window_update_entry_focus (context->window, ephy_embed_get_web_view (active_child));
-      gtk_widget_show (GTK_WIDGET (context->window));
+      gtk_widget_set_visible (GTK_WIDGET (context->window), TRUE);
     }
 
     ephy_embed_shell_restored_window (shell);
@@ -1794,7 +1768,7 @@ ephy_session_clear (EphySession *session)
   shell = ephy_shell_get_default ();
   windows = g_list_copy (gtk_application_get_windows (GTK_APPLICATION (shell)));
   for (p = windows; p; p = p->next)
-    gtk_widget_destroy (GTK_WIDGET (p->data));
+    gtk_window_destroy (GTK_WINDOW (p->data));
   g_list_free (windows);
   g_queue_foreach (session->closed_tabs,
                    (GFunc)closed_tab_free, NULL);
