@@ -29,7 +29,6 @@
 #include "ephy-embed-utils.h"
 #include "ephy-file-helpers.h"
 #include "ephy-firefox-sync-dialog.h"
-#include "ephy-gui.h"
 #include "ephy-header-bar.h"
 #include "ephy-history-dialog.h"
 #include "ephy-link.h"
@@ -50,7 +49,10 @@
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <handy.h>
+
+#ifdef USE_GRANITE
+#include <granite-7.h>
+#endif
 
 struct _EphyShell {
   EphyEmbedShell parent_instance;
@@ -69,10 +71,16 @@ struct _EphyShell {
   EphyShellStartupContext *local_startup_context;
   EphyShellStartupContext *remote_startup_context;
   GSList *open_uris_idle_ids;
+  EphyWebApplication *webapp;
 
-  GHashTable *notifications;
   gchar *open_notification_id;
   gboolean startup_finished;
+
+#if USE_GRANITE
+  GtkStyleProvider *style_provider;
+#endif
+
+  GList *windows; /* unowned, used for assertion only */
 };
 
 static EphyShell *ephy_shell = NULL;
@@ -80,7 +88,7 @@ static EphyShell *ephy_shell = NULL;
 static void ephy_shell_dispose (GObject *object);
 static void ephy_shell_finalize (GObject *object);
 
-G_DEFINE_TYPE (EphyShell, ephy_shell, EPHY_TYPE_EMBED_SHELL)
+G_DEFINE_FINAL_TYPE (EphyShell, ephy_shell, EPHY_TYPE_EMBED_SHELL)
 
 /**
  * ephy_shell_startup_context_new:
@@ -324,7 +332,7 @@ show_downloads (GSimpleAction *action,
   g_application_withdraw_notification (G_APPLICATION (ephy_shell), ephy_shell->open_notification_id);
   g_clear_pointer (&ephy_shell->open_notification_id, g_free);
 
-  gtk_widget_show (GTK_WIDGET (window));
+  gtk_widget_set_visible (GTK_WIDGET (window), TRUE);
   g_signal_emit_by_name (manager, "show-downloads", NULL);
 }
 
@@ -333,28 +341,27 @@ launch_app (GSimpleAction *action,
             GVariant      *parameter,
             gpointer       user_data)
 {
-  const gchar *desktop_file = g_variant_get_string (parameter, NULL);
+  const gchar *webapp_id = g_variant_get_string (parameter, NULL);
 
-  /* We can't get here under flatpak because all web app functionality
-   * is disabled when running under flatpak.
-   */
-  ephy_file_launch_desktop_file (desktop_file,
-                                 gtk_get_current_event_time (),
-                                 EPHY_FILE_HELPERS_I_UNDERSTAND_I_MUST_NOT_USE_THIS_FUNCTION_UNDER_FLATPAK);
+  ephy_web_application_launch (webapp_id);
 }
 
 static void
-notification_clicked (GSimpleAction *action,
-                      GVariant      *parameter,
-                      gpointer       user_data)
+webextension_action (GSimpleAction *action,
+                     GVariant      *parameter,
+                     gpointer       user_data)
 {
-  EphyShell *shell = ephy_shell_get_default ();
-  guint64 id = g_variant_get_uint64 (parameter);
-  WebKitNotification *notification;
+  EphyWebExtensionManager *manager = ephy_web_extension_manager_get_default ();
+  ephy_web_extension_manager_handle_notifications_action (manager, parameter);
+}
 
-  notification = g_hash_table_lookup (shell->notifications, GINT_TO_POINTER (id));
-  if (notification)
-    webkit_notification_clicked (notification);
+static void
+webextension_context_menu_action (GSimpleAction *action,
+                                  GVariant      *parameter,
+                                  gpointer       user_data)
+{
+  EphyWebExtensionManager *manager = ephy_web_extension_manager_get_default ();
+  ephy_web_extension_manager_handle_context_menu_action (manager, parameter);
 }
 
 static GActionEntry app_entries[] = {
@@ -372,7 +379,8 @@ static GActionEntry app_entries[] = {
   { "quit", quit_application, NULL, NULL, NULL },
   { "show-downloads", show_downloads, NULL, NULL, NULL },
   { "launch-app", launch_app, "s", NULL, NULL },
-  { "notification-clicked", notification_clicked, "t", NULL, NULL},
+  { "webextension-notification", webextension_action, "(ssi)", NULL, NULL },
+  { "webextension-context-menu", webextension_context_menu_action, "(sss)", NULL, NULL },
 };
 
 static GActionEntry non_incognito_extra_app_entries[] = {
@@ -380,12 +388,12 @@ static GActionEntry non_incognito_extra_app_entries[] = {
 };
 
 static GActionEntry app_mode_app_entries[] = {
+  { "new-window", new_window, NULL, NULL, NULL },
   { "history", show_history, NULL, NULL, NULL },
   { "preferences", show_preferences, NULL, NULL, NULL },
   { "about", show_about, NULL, NULL, NULL },
   { "quit", quit_application, NULL, NULL, NULL },
   { "run-in-background", NULL, NULL, "false", NULL},
-  { "notification-clicked", notification_clicked, "t", NULL, NULL},
 };
 
 static void
@@ -402,10 +410,12 @@ register_synchronizable_managers (EphyShell       *shell,
     ephy_sync_service_register_manager (service, manager);
   }
 
+/* https://gitlab.gnome.org/GNOME/epiphany/-/issues/1118#note_1731178
   if (ephy_sync_utils_bookmarks_sync_is_enabled ()) {
     manager = EPHY_SYNCHRONIZABLE_MANAGER (ephy_shell_get_bookmarks_manager (shell));
     ephy_sync_service_register_manager (service, manager);
   }
+*/
 
   if (ephy_sync_utils_passwords_sync_is_enabled ()) {
     manager = EPHY_SYNCHRONIZABLE_MANAGER (ephy_embed_shell_get_password_manager (EPHY_EMBED_SHELL (shell)));
@@ -493,19 +503,34 @@ ephy_shell_startup (GApplication *application)
 
   G_APPLICATION_CLASS (ephy_shell_parent_class)->startup (application);
 
-  hdy_init ();
-
-  hdy_style_manager_set_color_scheme (hdy_style_manager_get_default (),
-                                      HDY_COLOR_SCHEME_PREFER_LIGHT);
-
-  /* If we are under Pantheon set the icon-theme and cursor-theme accordingly. */
+#if USE_GRANITE
   if (is_desktop_pantheon ()) {
+    /* If we are under Pantheon set the icon-theme and cursor-theme accordingly. */
     GtkSettings *settings = gtk_settings_get_default ();
+    g_autofree char *theme_name = NULL;
+
     g_object_set (settings,
                   "gtk-icon-theme-name", "elementary",
                   "gtk-cursor-theme-name", "elementary",
                   NULL);
+
+    g_object_get (settings, "gtk-theme-name", &theme_name, NULL);
+
+    if (!g_str_has_prefix (theme_name, "io.elementary")) {
+      g_object_set (settings,
+                    "gtk-theme-name", "io.elementary.stylesheet.blueberry",
+                    NULL);
+    }
+
+    shell->style_provider = GTK_STYLE_PROVIDER (gtk_css_provider_new ());
+    gtk_css_provider_load_from_resource (GTK_CSS_PROVIDER (shell->style_provider),
+                                         "/org/gnome/Epiphany/style-elementary.css");
+
+    gtk_style_context_add_provider_for_display (gdk_display_get_default (),
+                                                shell->style_provider,
+                                                GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
   }
+#endif
 
   /* We're not remoting; start our services */
 
@@ -534,7 +559,6 @@ ephy_shell_startup (GApplication *application)
     }
 
     /* Actions that are disabled in app mode */
-    set_accel_for_action (shell, "app.new-window", "<Primary>n");
     set_accel_for_action (shell, "app.new-incognito", "<Primary><Shift>n");
     set_accel_for_action (shell, "app.reopen-closed-tab", "<Primary><Shift>t");
     set_accel_for_action (shell, "app.import-bookmarks", "<Primary><Shift>m");
@@ -542,6 +566,8 @@ ephy_shell_startup (GApplication *application)
     set_accel_for_action (shell, "app.shortcuts", "<Primary>question");
     set_accel_for_action (shell, "app.help", "F1");
   } else {
+    shell->webapp = ephy_web_application_for_profile_directory (ephy_profile_dir (), EPHY_WEB_APP_NO_TMP_ICON);
+
     g_action_map_add_action_entries (G_ACTION_MAP (application),
                                      app_mode_app_entries, G_N_ELEMENTS (app_mode_app_entries),
                                      application);
@@ -559,8 +585,9 @@ ephy_shell_startup (GApplication *application)
   }
 
   /* Actions that are available in both app mode and browser mode */
+  set_accel_for_action (shell, "app.new-window", "<Primary>n");
   set_accel_for_action (shell, "app.history", "<Primary>h");
-  set_accel_for_action (shell, "app.preferences", "<Primary>e");
+  set_accel_for_action (shell, "app.preferences", "<Primary>comma");
   set_accel_for_action (shell, "app.quit", "<Primary>q");
 }
 
@@ -818,11 +845,6 @@ ephy_shell_constructed (GObject *object)
     g_application_set_flags (G_APPLICATION (object), flags);
   }
 
-  if (ephy_embed_shell_get_mode (EPHY_EMBED_SHELL (object)) == EPHY_EMBED_SHELL_MODE_APPLICATION) {
-    dzl_application_add_resources (DZL_APPLICATION (object),
-                                   "resource:///org/gnome/Epiphany");
-  }
-
   /* FIXME: not sure if this is the best place to put this stuff. */
   ephy_shell_get_lockdown (EPHY_SHELL (object));
 
@@ -858,7 +880,10 @@ ephy_shell_init (EphyShell *shell)
   g_object_add_weak_pointer (G_OBJECT (ephy_shell),
                              (gpointer *)ptr);
 
-  ephy_shell->notifications = g_hash_table_new (g_direct_hash, g_direct_equal);
+#ifdef USE_GRANITE
+  if (is_desktop_pantheon ())
+    granite_init ();
+#endif
 }
 
 static void
@@ -867,6 +892,20 @@ remove_open_uris_idle_cb (gpointer data)
   g_source_remove (GPOINTER_TO_UINT (data));
 }
 
+void
+ephy_shell_register_window (EphyShell  *shell,
+                            EphyWindow *window)
+{
+  shell->windows = g_list_prepend (shell->windows, window);
+}
+
+void
+ephy_shell_unregister_window (EphyShell  *shell,
+                              EphyWindow *window)
+{
+  shell->windows = g_list_remove (shell->windows, window);
+}
+
 static void
 ephy_shell_dispose (GObject *object)
 {
@@ -874,9 +913,13 @@ ephy_shell_dispose (GObject *object)
 
   LOG ("EphyShell disposing");
 
+  if (shell->history_dialog) {
+    gtk_window_destroy (GTK_WINDOW (shell->history_dialog));
+    shell->history_dialog = NULL;
+  }
+
   g_clear_object (&shell->session);
   g_clear_object (&shell->lockdown);
-  g_clear_pointer (&shell->history_dialog, gtk_widget_destroy);
   g_clear_object (&shell->prefs_dialog);
   g_clear_object (&shell->network_monitor);
   g_clear_object (&shell->sync_service);
@@ -884,8 +927,7 @@ ephy_shell_dispose (GObject *object)
   g_clear_object (&shell->history_manager);
   g_clear_object (&shell->open_tabs_manager);
   g_clear_object (&shell->web_extension_manager);
-
-  g_hash_table_destroy (shell->notifications);
+  g_clear_pointer (&shell->webapp, ephy_web_application_free);
 
   if (shell->open_notification_id) {
     g_application_withdraw_notification (G_APPLICATION (shell), shell->open_notification_id);
@@ -895,6 +937,10 @@ ephy_shell_dispose (GObject *object)
   g_slist_free_full (shell->open_uris_idle_ids, remove_open_uris_idle_cb);
   shell->open_uris_idle_ids = NULL;
 
+#if USE_GRANITE
+  g_clear_object (&shell->style_provider);
+#endif
+
   G_OBJECT_CLASS (ephy_shell_parent_class)->dispose (object);
 }
 
@@ -906,6 +952,9 @@ ephy_shell_finalize (GObject *object)
   g_clear_pointer (&shell->local_startup_context, ephy_shell_startup_context_free);
   g_clear_pointer (&shell->remote_startup_context, ephy_shell_startup_context_free);
 
+  /* Ensure all windows have been destroyed. */
+  g_assert (!shell->windows);
+
   G_OBJECT_CLASS (ephy_shell_parent_class)->finalize (object);
 
   LOG ("Ephy shell finalised");
@@ -962,44 +1011,14 @@ webkit_notification_clicked_cb (WebKitNotification *notification,
   }
 }
 
-static void
-webkit_notification_closed_cb (WebKitNotification *notification,
-                               gpointer            user_data)
-{
-  EphyShell *shell = ephy_shell_get_default ();
-  g_autofree char *id = NULL;
-
-  id = g_strdup_printf ("%" G_GUINT64_FORMAT, webkit_notification_get_id (notification));
-
-  g_application_withdraw_notification (G_APPLICATION (g_application_get_default ()), id);
-  g_hash_table_remove (shell->notifications, GINT_TO_POINTER (webkit_notification_get_id (notification)));
-}
-
 static gboolean
 show_notification_cb (WebKitWebView      *web_view,
                       WebKitNotification *notification,
                       gpointer            user_data)
 {
-  EphyShell *shell = ephy_shell_get_default ();
-  GNotification *notify;
-  g_autofree char *id = NULL;
-
-  id = g_strdup_printf ("%" G_GUINT64_FORMAT, webkit_notification_get_id (notification));
-
   g_signal_connect_object (notification, "clicked", G_CALLBACK (webkit_notification_clicked_cb), web_view, 0);
-  g_signal_connect_object (notification, "closed", G_CALLBACK (webkit_notification_closed_cb), web_view, 0);
 
-  notify = g_notification_new (webkit_notification_get_title (notification));
-  g_notification_set_body (notify, webkit_notification_get_body (notification));
-
-  g_notification_set_priority (notify, G_NOTIFICATION_PRIORITY_LOW);
-  g_notification_set_default_action_and_target (notify, "app.notification-clicked", "t", webkit_notification_get_id (notification));
-
-  g_hash_table_insert (shell->notifications, GINT_TO_POINTER (webkit_notification_get_id (notification)), notification);
-
-  g_application_send_notification (G_APPLICATION (g_application_get_default ()), id, notify);
-
-  return TRUE;
+  return FALSE;
 }
 
 /**
@@ -1031,7 +1050,7 @@ ephy_shell_new_tab_full (EphyShell       *shell,
   EphyEmbed *parent = NULL;
 
   g_assert (EPHY_IS_SHELL (shell));
-  g_assert (EPHY_IS_WINDOW (window));
+  g_assert (EPHY_IS_WINDOW (window) || !window);
   g_assert (EPHY_IS_EMBED (previous_embed) || !previous_embed);
 
   embed_shell = EPHY_EMBED_SHELL (shell);
@@ -1039,6 +1058,9 @@ ephy_shell_new_tab_full (EphyShell       *shell,
   if (flags & EPHY_NEW_TAB_JUMP)
     jump_to = TRUE;
 
+  if (!window)
+    window = EPHY_WINDOW (gtk_application_get_active_window (GTK_APPLICATION (shell)));
+
   LOG ("Opening new tab window %p parent-embed %p jump-to:%s",
        window, previous_embed, jump_to ? "t" : "f");
 
@@ -1064,14 +1086,16 @@ ephy_shell_new_tab_full (EphyShell       *shell,
                                     "title", title,
                                     "progress-bar-enabled", ephy_embed_shell_get_mode (embed_shell) == EPHY_EMBED_SHELL_MODE_APPLICATION,
                                     NULL));
-  gtk_widget_show (GTK_WIDGET (embed));
   ephy_embed_container_add_child (EPHY_EMBED_CONTAINER (window), embed, parent, position, jump_to);
 
   if ((flags & EPHY_NEW_TAB_DONT_SHOW_WINDOW) == 0 &&
       ephy_embed_shell_get_mode (embed_shell) != EPHY_EMBED_SHELL_MODE_TEST) {
-    gtk_widget_show (GTK_WIDGET (window));
+    gtk_widget_set_visible (GTK_WIDGET (window), TRUE);
   }
 
+  if (shell->startup_finished && !jump_to)
+    ephy_window_switch_to_new_tab_toast (window, GTK_WIDGET (embed));
+
   return embed;
 }
 
@@ -1169,7 +1193,6 @@ ephy_shell_get_bookmarks_manager (EphyShell *shell)
 
   return shell->bookmarks_manager;
 }
-
 /**
  * ephy_shell_get_history_manager:
  * @shell: the #EphyShell
@@ -1220,6 +1243,14 @@ ephy_shell_get_net_monitor (EphyShell *shell)
   return shell->network_monitor;
 }
 
+static void
+window_destroyed (GtkWidget  *widget,
+                  GtkWidget **widget_pointer)
+{
+  if (widget_pointer)
+    *widget_pointer = NULL;
+}
+
 /**
  * ephy_shell_get_history_dialog:
  *
@@ -1238,7 +1269,7 @@ ephy_shell_get_history_dialog (EphyShell *shell)
     shell->history_dialog = ephy_history_dialog_new (service);
     g_signal_connect (shell->history_dialog,
                       "destroy",
-                      G_CALLBACK (gtk_widget_destroyed),
+                      G_CALLBACK (window_destroyed),
                       &shell->history_dialog);
   }
 
@@ -1257,7 +1288,7 @@ ephy_shell_get_firefox_sync_dialog (EphyShell *shell)
     shell->firefox_sync_dialog = ephy_firefox_sync_dialog_new ();
     g_signal_connect (shell->firefox_sync_dialog,
                       "destroy",
-                      G_CALLBACK (gtk_widget_destroyed),
+                      G_CALLBACK (window_destroyed),
                       &shell->firefox_sync_dialog);
   }
 
@@ -1277,7 +1308,7 @@ ephy_shell_get_prefs_dialog (EphyShell *shell)
 
     g_signal_connect (shell->prefs_dialog,
                       "destroy",
-                      G_CALLBACK (gtk_widget_destroyed),
+                      G_CALLBACK (window_destroyed),
                       &shell->prefs_dialog);
   }
 
@@ -1359,7 +1390,7 @@ ephy_shell_close_all_windows (EphyShell *shell)
     windows = windows->next;
 
     if (ephy_window_close (window))
-      gtk_widget_destroy (GTK_WIDGET (window));
+      gtk_window_destroy (GTK_WINDOW (window));
     else
       retval = FALSE;
   }
@@ -1443,6 +1474,7 @@ ephy_shell_open_uris_idle (OpenURIsData *data)
   EphyNewTabFlags page_flags = 0;
   gboolean reusing_empty_tab = FALSE;
   const char *url;
+  gboolean url_is_xpi;
 
   mode = ephy_embed_shell_get_mode (EPHY_EMBED_SHELL (data->shell));
 
@@ -1457,7 +1489,10 @@ ephy_shell_open_uris_idle (OpenURIsData *data)
       reusing_empty_tab = TRUE;
   }
 
-  if (!reusing_empty_tab) {
+  url = data->uris ? data->uris[data->current_uri] : NULL;
+  url_is_xpi = url && g_str_has_prefix (url, "file:") && g_str_has_suffix (url, ".xpi");
+
+  if (!reusing_empty_tab && !url_is_xpi) {
     embed = ephy_shell_new_tab_full (data->shell,
                                      NULL, NULL,
                                      data->window,
@@ -1466,8 +1501,10 @@ ephy_shell_open_uris_idle (OpenURIsData *data)
                                      data->user_time);
   }
 
-  url = data->uris ? data->uris[data->current_uri] : NULL;
-  if (url && url[0] != '\0') {
+  if (url_is_xpi) {
+    g_autoptr (GFile) xpi_file = g_file_new_for_uri (url);
+    ephy_web_extension_manager_install (ephy_web_extension_manager_get_default (), xpi_file);
+  } else if (url && url[0] != '\0') {
     ephy_web_view_load_url (ephy_embed_get_web_view (embed), url);
 
     /* When reusing an empty tab, the focus is in the location entry */
@@ -1482,10 +1519,12 @@ ephy_shell_open_uris_idle (OpenURIsData *data)
       ephy_window_activate_location (data->window);
   }
 
-  /* Set address from the very beginning. Looks odd in app mode if it appears later on. */
-  header_bar = EPHY_HEADER_BAR (ephy_window_get_header_bar (data->window));
-  title_widget = ephy_header_bar_get_title_widget (header_bar);
-  ephy_title_widget_set_address (title_widget, url);
+  if (!url_is_xpi) {
+    /* Set address from the very beginning. Looks odd in app mode if it appears later on. */
+    header_bar = EPHY_HEADER_BAR (ephy_window_get_header_bar (data->window));
+    title_widget = ephy_header_bar_get_title_widget (header_bar);
+    ephy_title_widget_set_address (title_widget, url);
+  }
 
   data->current_uri++;
   data->previous_embed = embed;
@@ -1542,18 +1581,6 @@ ephy_shell_startup_finished (EphyShell *shell)
   return shell->startup_finished;
 }
 
-EphyWebExtensionManager *
-ephy_shell_get_web_extension_manager (EphyShell *shell)
-{
-  g_assert (EPHY_IS_SHELL (shell));
-
-  if (shell->web_extension_manager == NULL)
-    shell->web_extension_manager = ephy_web_extension_manager_new ();
-
-  return shell->web_extension_manager;
-}
-
-
 /* Helper functions: better place for this? */
 EphyWebView *
 ephy_shell_get_web_view (EphyShell *shell,
@@ -1597,3 +1624,37 @@ ephy_shell_get_active_web_view (EphyShell *shell)
 
   return ephy_embed_get_web_view (EPHY_EMBED (page));
 }
+
+EphyWebApplication *
+ephy_shell_get_webapp (EphyShell *shell)
+{
+  return shell->webapp;
+}
+
+void
+ephy_shell_resync_title_boxes (EphyShell  *shell,
+                               const char *title,
+                               const char *address)
+{
+  GList *windows;
+  EphyEmbedShellMode mode;
+
+  mode = ephy_embed_shell_get_mode (EPHY_EMBED_SHELL (shell));
+  g_assert (mode == EPHY_EMBED_SHELL_MODE_APPLICATION);
+
+  windows = gtk_application_get_windows (GTK_APPLICATION (shell));
+  while (windows) {
+    EphyWindow *window = EPHY_WINDOW (windows->data);
+    EphyHeaderBar *header_bar;
+    EphyTitleBox *title_box;
+
+    header_bar = EPHY_HEADER_BAR (ephy_window_get_header_bar (window));
+    title_box = EPHY_TITLE_BOX (ephy_header_bar_get_title_widget (header_bar));
+    g_assert (EPHY_IS_TITLE_BOX (title_box));
+
+    ephy_title_box_reset (title_box, title, address);
+    gtk_window_set_title (GTK_WINDOW (window), title);
+
+    windows = windows->next;
+  }
+}
