@@ -21,14 +21,12 @@
 #include "config.h"
 #include "ephy-data-view.h"
 
+#include <adwaita.h>
 #include <ctype.h>
-#include <handy.h>
 
 typedef struct {
   GtkWidget *box;
   GtkWidget *child;
-  GtkWidget *header_bar;
-  GtkWidget *back_button;
   GtkWidget *clear_button;
   GtkWidget *search_bar;
   GtkWidget *search_entry;
@@ -42,11 +40,12 @@ typedef struct {
   gboolean has_search_results : 1;
   gboolean can_clear : 1;
   char *search_text;
+  char *search_description;
 } EphyDataViewPrivate;
 
 static void ephy_data_view_buildable_init (GtkBuildableIface *iface);
 
-G_DEFINE_TYPE_WITH_CODE (EphyDataView, ephy_data_view, GTK_TYPE_BIN,
+G_DEFINE_TYPE_WITH_CODE (EphyDataView, ephy_data_view, ADW_TYPE_NAVIGATION_PAGE,
                          G_ADD_PRIVATE (EphyDataView)
                          G_IMPLEMENT_INTERFACE (GTK_TYPE_BUILDABLE,
                                                 ephy_data_view_buildable_init))
@@ -55,7 +54,6 @@ static GtkBuildableIface *parent_buildable_iface;
 
 enum {
   PROP_0,
-  PROP_TITLE,
   PROP_CLEAR_ACTION_NAME,
   PROP_CLEAR_ACTION_TARGET,
   PROP_CLEAR_BUTTON_LABEL,
@@ -74,7 +72,6 @@ enum {
 static GParamSpec *obj_properties[LAST_PROP];
 
 enum {
-  BACK_BUTTON_CLICKED,
   CLEAR_BUTTON_CLICKED,
   LAST_SIGNAL,
 };
@@ -109,13 +106,6 @@ update (EphyDataView *self)
   gtk_widget_set_sensitive (priv->search_button, has_data);
 }
 
-static void
-on_back_button_clicked (GtkWidget    *button,
-                        EphyDataView *self)
-{
-  g_signal_emit (self, signals[BACK_BUTTON_CLICKED], 0);
-}
-
 static void
 on_clear_button_clicked (EphyDataView *self)
 {
@@ -129,55 +119,13 @@ on_search_entry_changed (GtkSearchEntry *entry,
   EphyDataViewPrivate *priv = ephy_data_view_get_instance_private (self);
   const char *text;
 
-  text = gtk_entry_get_text (GTK_ENTRY (entry));
+  text = gtk_editable_get_text (GTK_EDITABLE (entry));
   g_free (priv->search_text);
   priv->search_text = g_strdup (text);
 
   g_object_notify_by_pspec (G_OBJECT (self), obj_properties[PROP_SEARCH_TEXT]);
 }
 
-gboolean
-ephy_data_view_handle_event (EphyDataView *self,
-                             GdkEvent     *event)
-{
-  EphyDataViewPrivate *priv = ephy_data_view_get_instance_private (self);
-  GdkEventKey *key = (GdkEventKey *)event;
-  gint result;
-
-  /* Firstly, we check if the HdySearchBar can handle the event */
-  result = hdy_search_bar_handle_event (HDY_SEARCH_BAR (priv->search_bar), event);
-
-  if (result == GDK_EVENT_STOP)
-    return result;
-
-  /* Secondly, we check for the shortcuts implemented by the data views */
-  /* Ctrl + F */
-  if ((key->state & GDK_CONTROL_MASK) && key->keyval == GDK_KEY_f) {
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->search_button), TRUE);
-    return GDK_EVENT_STOP;
-  }
-
-  /* Shift + Delete */
-  if ((key->state & GDK_SHIFT_MASK) && key->keyval == GDK_KEY_Delete) {
-    gtk_button_clicked (GTK_BUTTON (priv->clear_button));
-    return GDK_EVENT_STOP;
-  }
-
-  /* Finally, we check for the Escape key */
-  if (key->keyval == GDK_KEY_Escape) {
-    if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->search_button)))
-      /* If the user presses the Escape key and the search bar is hidden,
-       * we want the event to have the same effect as clicking the back button*/
-      gtk_button_clicked (GTK_BUTTON (priv->back_button));
-    else
-      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->search_button), FALSE);
-
-    return GDK_EVENT_STOP;
-  }
-
-  return GDK_EVENT_PROPAGATE;
-}
-
 static void
 ephy_data_view_set_property (GObject      *object,
                              guint         prop_id,
@@ -188,9 +136,6 @@ ephy_data_view_set_property (GObject      *object,
   EphyDataViewPrivate *priv = ephy_data_view_get_instance_private (self);
 
   switch (prop_id) {
-    case PROP_TITLE:
-      gtk_header_bar_set_title (GTK_HEADER_BAR (priv->header_bar), g_value_get_string (value));
-      break;
     case PROP_CLEAR_ACTION_NAME:
       gtk_actionable_set_action_name (GTK_ACTIONABLE (priv->clear_button), g_value_get_string (value));
       break;
@@ -204,14 +149,18 @@ ephy_data_view_set_property (GObject      *object,
       ephy_data_view_set_clear_button_tooltip (self, g_value_get_string (value));
       break;
     case PROP_SEARCH_DESCRIPTION:
-      gtk_entry_set_placeholder_text (GTK_ENTRY (priv->search_entry), g_value_get_string (value));
-      atk_object_set_description (gtk_widget_get_accessible (GTK_WIDGET (self)), g_value_get_string (value));
+      g_clear_pointer (&priv->search_description, g_free);
+      priv->search_description = g_value_dup_string (value);
+      gtk_accessible_update_property (GTK_ACCESSIBLE (self),
+                                      GTK_ACCESSIBLE_PROPERTY_DESCRIPTION,
+                                      priv->search_description,
+                                      -1);
       break;
     case PROP_EMPTY_TITLE:
-      hdy_status_page_set_title (HDY_STATUS_PAGE (priv->empty_page), g_value_get_string (value));
+      adw_status_page_set_title (ADW_STATUS_PAGE (priv->empty_page), g_value_get_string (value));
       break;
     case PROP_EMPTY_DESCRIPTION:
-      hdy_status_page_set_description (HDY_STATUS_PAGE (priv->empty_page), g_value_get_string (value));
+      adw_status_page_set_description (ADW_STATUS_PAGE (priv->empty_page), g_value_get_string (value));
       break;
     case PROP_IS_LOADING:
       ephy_data_view_set_is_loading (self, g_value_get_boolean (value));
@@ -241,9 +190,6 @@ ephy_data_view_get_property (GObject    *object,
   EphyDataViewPrivate *priv = ephy_data_view_get_instance_private (self);
 
   switch (prop_id) {
-    case PROP_TITLE:
-      g_value_set_string (value, gtk_header_bar_get_title (GTK_HEADER_BAR (priv->header_bar)));
-      break;
     case PROP_CLEAR_ACTION_NAME:
       g_value_set_string (value, gtk_actionable_get_action_name (GTK_ACTIONABLE (priv->clear_button)));
       break;
@@ -257,13 +203,13 @@ ephy_data_view_get_property (GObject    *object,
       g_value_set_string (value, ephy_data_view_get_clear_button_tooltip (self));
       break;
     case PROP_SEARCH_DESCRIPTION:
-      g_value_set_string (value, gtk_entry_get_placeholder_text (GTK_ENTRY (priv->search_entry)));
+      g_value_set_string (value, priv->search_description);
       break;
     case PROP_EMPTY_TITLE:
-      g_value_set_string (value, hdy_status_page_get_title (HDY_STATUS_PAGE (priv->empty_page)));
+      g_value_set_string (value, adw_status_page_get_title (ADW_STATUS_PAGE (priv->empty_page)));
       break;
     case PROP_EMPTY_DESCRIPTION:
-      g_value_set_string (value, hdy_status_page_get_description (HDY_STATUS_PAGE (priv->empty_page)));
+      g_value_set_string (value, adw_status_page_get_description (ADW_STATUS_PAGE (priv->empty_page)));
       break;
     case PROP_SEARCH_TEXT:
       g_value_set_string (value, ephy_data_view_get_search_text (self));
@@ -293,10 +239,24 @@ ephy_data_view_finalize (GObject *object)
   EphyDataViewPrivate *priv = ephy_data_view_get_instance_private (self);
 
   g_free (priv->search_text);
+  g_free (priv->search_description);
 
   G_OBJECT_CLASS (ephy_data_view_parent_class)->finalize (object);
 }
 
+static gboolean
+find_shortuct_cb (EphyDataView *self)
+{
+  EphyDataViewPrivate *priv = ephy_data_view_get_instance_private (self);
+  gboolean search;
+
+  search = gtk_search_bar_get_search_mode (GTK_SEARCH_BAR (priv->search_bar));
+
+  gtk_search_bar_set_search_mode (GTK_SEARCH_BAR (priv->search_bar), !search);
+
+  return GDK_EVENT_STOP;
+}
+
 static void
 ephy_data_view_class_init (EphyDataViewClass *klass)
 {
@@ -307,106 +267,80 @@ ephy_data_view_class_init (EphyDataViewClass *klass)
   object_class->get_property = ephy_data_view_get_property;
   object_class->finalize = ephy_data_view_finalize;
 
-  obj_properties[PROP_TITLE] =
-    g_param_spec_string ("title",
-                         "Title",
-                         "The title used for the header bar",
-                         NULL,
-                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
-
   obj_properties[PROP_CLEAR_ACTION_NAME] =
     g_param_spec_string ("clear-action-name",
-                         "'Clear' action name",
-                         "The name of the action associated to the 'Clear' button",
+                         NULL, NULL,
                          NULL,
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
   obj_properties[PROP_CLEAR_ACTION_TARGET] =
     g_param_spec_variant ("clear-action-target",
-                          "'Clear' action target value",
-                          "The parameter for 'Clear' action invocations",
+                          NULL, NULL,
                           G_VARIANT_TYPE_ANY, NULL,
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
   obj_properties[PROP_CLEAR_BUTTON_LABEL] =
     g_param_spec_string ("clear-button-label",
-                         "'Clear' button label",
-                         "The label of the 'Clear' button",
+                         NULL, NULL,
                          NULL,
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
   obj_properties[PROP_CLEAR_BUTTON_TOOLTIP] =
     g_param_spec_string ("clear-button-tooltip",
-                         "'Clear' button tooltip",
-                         "The description of the 'Clear' action",
+                         NULL, NULL,
                          NULL,
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
   obj_properties[PROP_SEARCH_DESCRIPTION] =
     g_param_spec_string ("search-description",
-                         "'Search' description",
-                         "The description of the 'Search' action",
+                         NULL, NULL,
                          NULL,
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
   obj_properties[PROP_EMPTY_TITLE] =
     g_param_spec_string ("empty-title",
-                         "'Empty' title",
-                         "The title of the 'Empty' state page",
+                         NULL, NULL,
                          NULL,
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
   obj_properties[PROP_EMPTY_DESCRIPTION] =
     g_param_spec_string ("empty-description",
-                         "'Empty' description",
-                         "The description of the 'Empty' state page",
+                         NULL, NULL,
                          NULL,
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
   obj_properties[PROP_SEARCH_TEXT] =
     g_param_spec_string ("search-text",
-                         "Search text",
-                         "The text of the search entry",
+                         NULL, NULL,
                          NULL,
                          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   obj_properties[PROP_IS_LOADING] =
     g_param_spec_boolean ("is-loading",
-                          "Is loading",
-                          "Whether the dialog is loading its data",
+                          NULL, NULL,
                           FALSE,
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
   obj_properties[PROP_HAS_DATA] =
     g_param_spec_boolean ("has-data",
-                          "Has data",
-                          "Whether the dialog has data",
+                          NULL, NULL,
                           FALSE,
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
   obj_properties[PROP_HAS_SEARCH_RESULTS] =
     g_param_spec_boolean ("has-search-results",
-                          "Has search results",
-                          "Whether the dialog has search results",
+                          NULL, NULL,
                           FALSE,
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
   obj_properties[PROP_CAN_CLEAR] =
     g_param_spec_boolean ("can-clear",
-                          "Can clear",
-                          "Whether the data can be cleared",
+                          NULL, NULL,
                           FALSE,
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
   g_object_class_install_properties (object_class, LAST_PROP, obj_properties);
 
-  signals[BACK_BUTTON_CLICKED] =
-    g_signal_new ("back-button-clicked",
-                  EPHY_TYPE_DATA_VIEW,
-                  G_SIGNAL_RUN_LAST,
-                  0, NULL, NULL, NULL,
-                  G_TYPE_NONE, 0);
-
   /**
    * EphyLocationEntry::user-changed:
    * @entry: the object on which the signal is emitted
@@ -427,8 +361,6 @@ ephy_data_view_class_init (EphyDataViewClass *klass)
                                                "/org/gnome/epiphany/gtk/data-view.ui");
 
   gtk_widget_class_bind_template_child_private (widget_class, EphyDataView, box);
-  gtk_widget_class_bind_template_child_private (widget_class, EphyDataView, header_bar);
-  gtk_widget_class_bind_template_child_private (widget_class, EphyDataView, back_button);
   gtk_widget_class_bind_template_child_private (widget_class, EphyDataView, clear_button);
   gtk_widget_class_bind_template_child_private (widget_class, EphyDataView, empty_page);
   gtk_widget_class_bind_template_child_private (widget_class, EphyDataView, search_bar);
@@ -437,9 +369,13 @@ ephy_data_view_class_init (EphyDataViewClass *klass)
   gtk_widget_class_bind_template_child_private (widget_class, EphyDataView, spinner);
   gtk_widget_class_bind_template_child_private (widget_class, EphyDataView, stack);
 
-  gtk_widget_class_bind_template_callback (widget_class, on_back_button_clicked);
   gtk_widget_class_bind_template_callback (widget_class, on_clear_button_clicked);
   gtk_widget_class_bind_template_callback (widget_class, on_search_entry_changed);
+
+  gtk_widget_class_add_binding (widget_class, GDK_KEY_F, GDK_CONTROL_MASK,
+                                (GtkShortcutFunc)find_shortuct_cb, NULL);
+  gtk_widget_class_add_binding_signal (widget_class, GDK_KEY_Delete, GDK_SHIFT_MASK,
+                                       "clear-button-clicked", NULL);
 }
 
 static void
@@ -449,9 +385,10 @@ ephy_data_view_init (EphyDataView *self)
 
   gtk_widget_init_template (GTK_WIDGET (self));
 
-  hdy_search_bar_connect_entry (HDY_SEARCH_BAR (priv->search_bar), GTK_ENTRY (priv->search_entry));
+  gtk_search_bar_connect_entry (GTK_SEARCH_BAR (priv->search_bar),
+                                GTK_EDITABLE (priv->search_entry));
 
-  hdy_status_page_set_icon_name (HDY_STATUS_PAGE (priv->empty_page),
+  adw_status_page_set_icon_name (ADW_STATUS_PAGE (priv->empty_page),
                                  APPLICATION_ID "-symbolic");
 
   update (self);
@@ -474,7 +411,7 @@ ephy_data_view_add_child (GtkBuildable *buildable,
   g_assert (!priv->child);
 
   priv->child = GTK_WIDGET (child);
-  gtk_container_add (GTK_CONTAINER (priv->stack), priv->child);
+  gtk_stack_add_child (GTK_STACK (priv->stack), priv->child);
 
   update (self);
 }
