@@ -62,7 +62,6 @@ typedef enum {
   COMMAND_NONE,
   COMMAND_BROWSER_ACTION,
   COMMAND_PAGE_ACTION,
-  COMMAND_SIDEBAR_ACTION,
 } Command;
 
 typedef struct {
@@ -128,8 +127,6 @@ get_command_property (JsonObject *object)
     return COMMAND_BROWSER_ACTION;
   if (strcmp (command, "_execute_page_action") == 0)
     return COMMAND_PAGE_ACTION;
-  if (strcmp (command, "_execute_sidebar_action") == 0)
-    return COMMAND_SIDEBAR_ACTION;
 
   return COMMAND_NONE;
 }
@@ -449,7 +446,7 @@ builder_add_modifier_array (JsonBuilder     *builder,
     json_builder_add_string_value (builder, "Ctrl");
   if (state & GDK_SHIFT_MASK)
     json_builder_add_string_value (builder, "Shift");
-  if (state & GDK_MOD1_MASK)
+  if (state & GDK_ALT_MASK)
     json_builder_add_string_value (builder, "Alt");
   json_builder_end_array (builder);
 }
@@ -556,6 +553,42 @@ rules_match_uri (GStrv  rules,
   return FALSE;
 }
 
+void
+menu_activate_browser_action (gpointer user_data)
+{
+  EphyWebExtension *web_extension = user_data;
+  EphyWebExtensionManager *manager = ephy_web_extension_manager_get_default ();
+
+  ephy_web_extension_manager_show_browser_action (manager, web_extension);
+}
+
+void
+menu_activate_page_button (gpointer user_data)
+{
+  EphyWebExtension *web_extension = user_data;
+  EphyWebExtensionManager *manager = ephy_web_extension_manager_get_default ();
+  EphyShell *shell = ephy_shell_get_default ();
+  EphyWebView *view = EPHY_WEB_VIEW (ephy_shell_get_active_web_view (shell));
+  GtkWidget *button = ephy_web_extension_manager_get_page_action (manager, web_extension, view);
+
+  gtk_widget_mnemonic_activate (button, false);
+}
+
+void
+menu_activate_command_action (GAction  *action,
+                              GVariant *params,
+                              gpointer  user_data)
+{
+  EphyWebExtension *web_extension = user_data;
+  Command command = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (action), "command"));
+
+  if (command == COMMAND_BROWSER_ACTION) {
+    g_idle_add_once (menu_activate_browser_action, web_extension);
+  } else if (command == COMMAND_PAGE_ACTION) {
+    g_idle_add_once (menu_activate_page_button, web_extension);
+  }
+}
+
 static WebKitContextMenuItem *
 create_context_menu_item (GHashTable          *menus,
                           const char          *name,
@@ -625,6 +658,13 @@ create_context_menu_item (GHashTable          *menus,
                                                                             tab_data));
     }
 
+    if (item->command != COMMAND_NONE) {
+      g_object_set_data (G_OBJECT (action), "command", GINT_TO_POINTER (item->command));
+      g_signal_connect (action, "activate",
+                        G_CALLBACK (menu_activate_command_action),
+                        self);
+    }
+
     menu_items = g_list_append (menu_items, menu_item);
   }
 
@@ -657,9 +697,9 @@ ephy_web_extension_api_menus_create_context_menu (EphyWebExtension    *self,
   action = g_action_map_lookup_action (G_ACTION_MAP (ephy_shell_get_default ()), "webextension-context-menu");
   g_assert (action);
 
-  document_uri = g_uri_parse (webkit_web_view_get_uri (web_view), G_URI_FLAGS_ENCODED_PATH | G_URI_FLAGS_ENCODED_QUERY | G_URI_FLAGS_SCHEME_NORMALIZE, NULL);
+  document_uri = g_uri_parse (webkit_web_view_get_uri (web_view), G_URI_FLAGS_PARSE_RELAXED | G_URI_FLAGS_ENCODED_PATH | G_URI_FLAGS_ENCODED_QUERY | G_URI_FLAGS_SCHEME_NORMALIZE, NULL);
   if (webkit_hit_test_result_get_link_uri (hit_test_result))
-    target_uri = g_uri_parse (webkit_hit_test_result_get_link_uri (hit_test_result), G_URI_FLAGS_ENCODED_PATH | G_URI_FLAGS_ENCODED_QUERY | G_URI_FLAGS_SCHEME_NORMALIZE, NULL);
+    target_uri = g_uri_parse (webkit_hit_test_result_get_link_uri (hit_test_result), G_URI_FLAGS_PARSE_RELAXED | G_URI_FLAGS_ENCODED_PATH | G_URI_FLAGS_ENCODED_QUERY | G_URI_FLAGS_SCHEME_NORMALIZE, NULL);
 
   return create_context_menu_item (menus, ephy_web_extension_get_short_name (self), self, web_view, modifiers,
                                    context_menu, hit_test_result, action, is_audio, is_video, is_editable,
