@@ -74,7 +74,7 @@ add_web_view_to_json (EphyWebExtension *extension,
   EphyTabView *tab_view = ephy_window_get_tab_view (window);
   GtkWidget *page = gtk_widget_get_parent (gtk_widget_get_parent (GTK_WIDGET (web_view)));
   gboolean is_active = ephy_tab_view_get_current_page (tab_view) == page;
-  WebKitFaviconDatabase *favicon_db = webkit_web_context_get_favicon_database (webkit_web_view_get_context (WEBKIT_WEB_VIEW (web_view)));
+  WebKitFaviconDatabase *favicon_db = ephy_embed_shell_get_favicon_database (ephy_embed_shell_get_default ());
   const char *favicon_uri = webkit_favicon_database_get_favicon_uri (favicon_db, ephy_web_view_get_address (web_view));
   gboolean has_tab_permission = ephy_web_extension_has_tab_or_host_permission (extension, web_view, TRUE);
 
@@ -135,7 +135,7 @@ ephy_web_extension_api_tabs_create_tab_object (EphyWebExtension *extension,
   g_autoptr (JsonBuilder) builder = json_builder_new ();
   add_web_view_to_json (extension,
                         builder,
-                        EPHY_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (web_view))),
+                        EPHY_WINDOW (gtk_widget_get_root (GTK_WIDGET (web_view))),
                         web_view);
   return json_builder_get_root (builder);
 }
@@ -294,7 +294,7 @@ tabs_handler_insert_css (EphyWebExtensionSender *sender,
     return;
   }
 
-  /* FIXME: Supoprt allFrames and cssOrigin */
+  /* FIXME: Support allFrames and cssOrigin */
   css = ephy_web_extension_add_custom_css (sender->extension, code);
   webkit_user_content_manager_add_style_sheet (ucm, css);
 
@@ -392,20 +392,20 @@ on_execute_script_ready (GObject      *source,
                          GAsyncResult *result,
                          gpointer      user_data)
 {
-  g_autoptr (WebKitJavascriptResult) js_result = NULL;
+  g_autoptr (JSCValue) value = NULL;
   g_autoptr (GError) error = NULL;
   GTask *task = user_data;
 
-  js_result = webkit_web_view_run_javascript_in_world_finish (WEBKIT_WEB_VIEW (source),
-                                                              result,
-                                                              &error);
+  value = webkit_web_view_evaluate_javascript_finish (WEBKIT_WEB_VIEW (source),
+                                                      result,
+                                                      &error);
 
   if (error) {
     g_task_return_error (task, g_steal_pointer (&error));
     return;
   }
 
-  g_task_return_pointer (task, jsc_value_to_json (webkit_javascript_result_get_js_value (js_result), 0), g_free);
+  g_task_return_pointer (task, jsc_value_to_json (value, 0), g_free);
 }
 
 static void
@@ -459,12 +459,13 @@ tabs_handler_execute_script (EphyWebExtensionSender *sender,
     return;
   }
 
-  webkit_web_view_run_javascript_in_world (target_web_view,
-                                           code,
-                                           ephy_web_extension_get_guid (sender->extension),
-                                           NULL,
-                                           on_execute_script_ready,
-                                           task);
+  webkit_web_view_evaluate_javascript (target_web_view,
+                                       code, -1,
+                                       ephy_web_extension_get_guid (sender->extension),
+                                       NULL,
+                                       NULL,
+                                       on_execute_script_ready,
+                                       task);
 }
 
 static void
